"""
Workflow Agents 异常定义

提供清晰、结构化的异常类型，帮助用户快速定位和修复 workflow YAML 配置问题。
所有错误消息支持国际化。
"""

from autocoder.common.international import get_message


class WorkflowError(Exception):
    """Workflow 基础异常类"""

    def __init__(self, message: str, details: dict = None):
        super().__init__(message)
        self.message = message
        self.details = details or {}


class WorkflowValidationError(WorkflowError):
    """Workflow 验证错误

    用于 YAML 配置格式验证、必需字段检查等场景。
    """

    def __init__(
        self,
        message: str,
        field_path: str = None,
        expected: str = None,
        actual: str = None,
        suggestion: str = None,
    ):
        """
        Args:
            message: 错误消息
            field_path: 字段路径，如 "spec.agents[0].id"
            expected: 期望的值或格式
            actual: 实际的值
            suggestion: 修复建议
        """
        details = {
            "field_path": field_path,
            "expected": expected,
            "actual": actual,
            "suggestion": suggestion,
        }
        # 过滤掉 None 值
        details = {k: v for k, v in details.items() if v is not None}

        super().__init__(message, details)
        self.field_path = field_path
        self.expected = expected
        self.actual = actual
        self.suggestion = suggestion

    def __str__(self):
        title = get_message("workflow.validation_error_title")
        lines = [f"{title}: {self.message}"]

        if self.field_path:
            field_label = get_message("workflow.field_path")
            lines.append(f"   {field_label}: {self.field_path}")

        if self.expected:
            expected_label = get_message("workflow.expected_value")
            lines.append(f"   {expected_label}: {self.expected}")

        if self.actual:
            actual_label = get_message("workflow.actual_value")
            lines.append(f"   {actual_label}: {self.actual}")

        if self.suggestion:
            suggestion_label = get_message("workflow.fix_suggestion")
            lines.append(f"   {suggestion_label}: {self.suggestion}")

        return "\n".join(lines)


class WorkflowFileNotFoundError(WorkflowError):
    """Workflow 文件未找到错误"""

    def __init__(self, workflow_name: str, searched_paths: list = None):
        self.workflow_name = workflow_name
        self.searched_paths = searched_paths or []

        message = f"未找到 workflow: '{workflow_name}'"
        details = {"workflow_name": workflow_name, "searched_paths": searched_paths}

        super().__init__(message, details)

    def __str__(self):
        title = get_message("workflow.file_not_found_title")
        lines = [f"{title}: {self.workflow_name}"]

        if self.searched_paths:
            searched_label = get_message("workflow.searched_paths")
            lines.append(f"   {searched_label}:")
            for path in self.searched_paths:
                lines.append(f"     - {path}")

        lines.append("")
        please_check = get_message("workflow.please_check")
        lines.append(f"   {please_check}:")
        check_name = get_message("workflow.check_name_correct")
        lines.append(f"     {check_name}")
        check_file = get_message("workflow.check_file_exists")
        lines.append(f"     {check_file}:")
        lines.append("        - .autocoderworkflow/")
        lines.append("        - .auto-coder/.autocoderworkflow/")
        lines.append("        - ~/.auto-coder/.autocoderworkflow/")

        return "\n".join(lines)


class WorkflowParseError(WorkflowError):
    """Workflow YAML 解析错误"""

    def __init__(self, yaml_path: str, parse_error: Exception, line_number: int = None):
        self.yaml_path = yaml_path
        self.parse_error = parse_error
        self.line_number = line_number

        message = f"解析 workflow YAML 文件失败: {yaml_path}"
        details = {
            "yaml_path": yaml_path,
            "error_type": type(parse_error).__name__,
            "error_message": str(parse_error),
            "line_number": line_number,
        }

        super().__init__(message, details)

    def __str__(self):
        title = get_message("workflow.parse_error_title")
        lines = [title]

        file_label = get_message("workflow.file")
        lines.append(f"   {file_label}: {self.yaml_path}")

        if self.line_number:
            line_label = get_message("workflow.line_number")
            lines.append(f"   {line_label}: {self.line_number}")

        error_type_label = get_message("workflow.error_type")
        lines.append(f"   {error_type_label}: {self.details['error_type']}")

        error_details_label = get_message("workflow.error_details")
        lines.append(f"   {error_details_label}: {self.details['error_message']}")

        lines.append("")
        common_issues = get_message("workflow.common_issues")
        lines.append(f"   {common_issues}:")

        check_syntax = get_message("workflow.check_yaml_syntax")
        lines.append(f"     {check_syntax}")

        no_tabs = get_message("workflow.no_tab_characters")
        lines.append(f"     {no_tabs}")

        check_chars = get_message("workflow.check_special_chars")
        lines.append(f"     {check_chars}")

        return "\n".join(lines)


class WorkflowStepError(WorkflowError):
    """Workflow 步骤执行错误"""

    def __init__(
        self,
        step_id: str,
        agent_id: str = None,
        error_message: str = None,
        cause: Exception = None,
    ):
        self.step_id = step_id
        self.agent_id = agent_id
        self.cause = cause

        message = f"步骤 '{step_id}' 执行失败"
        if error_message:
            message = f"{message}: {error_message}"

        details = {
            "step_id": step_id,
            "agent_id": agent_id,
            "error_message": error_message,
            "cause_type": type(cause).__name__ if cause else None,
        }

        super().__init__(message, details)

    def __str__(self):
        title = get_message("workflow.step_error_title")
        lines = [title]

        step_label = get_message("workflow.step_id")
        lines.append(f"   {step_label}: {self.step_id}")

        if self.agent_id:
            agent_label = get_message("workflow.agent_id")
            lines.append(f"   {agent_label}: {self.agent_id}")

        if self.details.get("error_message"):
            error_label = get_message("workflow.error")
            lines.append(f"   {error_label}: {self.details['error_message']}")

        if self.cause:
            cause_type_label = get_message("workflow.cause_type")
            lines.append(f"   {cause_type_label}: {self.details['cause_type']}")

            cause_details_label = get_message("workflow.cause_details")
            lines.append(f"   {cause_details_label}: {str(self.cause)}")

        return "\n".join(lines)


class WorkflowDependencyError(WorkflowError):
    """Workflow 依赖错误（循环依赖、缺失依赖等）"""

    def __init__(
        self, message: str, step_id: str = None, dependency_chain: list = None
    ):
        self.step_id = step_id
        self.dependency_chain = dependency_chain or []

        details = {"step_id": step_id, "dependency_chain": dependency_chain}

        super().__init__(message, details)

    def __str__(self):
        title = get_message("workflow.dependency_error_title")
        lines = [f"{title}: {self.message}"]

        if self.step_id:
            involved_label = get_message("workflow.involved_step")
            lines.append(f"   {involved_label}: {self.step_id}")

        if self.dependency_chain:
            chain_label = get_message("workflow.dependency_chain")
            lines.append(f"   {chain_label}:")
            for i, step in enumerate(self.dependency_chain):
                lines.append(f"     {i + 1}. {step}")

        lines.append("")
        suggestion_label = get_message("workflow.fix_suggestion")
        lines.append(f"   {suggestion_label}:")

        check_needs = get_message("workflow.check_needs_field")
        lines.append(f"     {check_needs}")

        no_circular = get_message("workflow.no_circular_dependency")
        lines.append(f"     {no_circular}")

        check_spelling = get_message("workflow.check_step_id_spelling")
        lines.append(f"     {check_spelling}")

        return "\n".join(lines)


class WorkflowAgentNotFoundError(WorkflowError):
    """Workflow 中引用的 agent 不存在"""

    def __init__(
        self, agent_id: str, step_id: str = None, available_agents: list = None
    ):
        self.agent_id = agent_id
        self.step_id = step_id
        self.available_agents = available_agents or []

        message = f"步骤引用的代理不存在: '{agent_id}'"
        if step_id:
            message = f"步骤 '{step_id}' {message}"

        details = {
            "agent_id": agent_id,
            "step_id": step_id,
            "available_agents": available_agents,
        }

        super().__init__(message, details)

    def __str__(self):
        title = get_message("workflow.agent_not_found_title")
        lines = [title]

        referenced_label = get_message("workflow.referenced_agent")
        lines.append(f"   {referenced_label}: {self.agent_id}")

        if self.step_id:
            step_label = get_message("workflow.in_step")
            lines.append(f"   {step_label}: {self.step_id}")

        available_label = get_message("workflow.available_agents")
        if self.available_agents:
            lines.append(f"   {available_label}:")
            for agent in self.available_agents:
                lines.append(f"     - {agent}")
        else:
            no_agents = get_message("workflow.no_agents")
            lines.append(f"   {available_label}: {no_agents}")

        lines.append("")
        suggestion_label = get_message("workflow.fix_suggestion")
        lines.append(f"   {suggestion_label}:")

        check_spelling = get_message("workflow.check_agent_spelling")
        lines.append(f"     {check_spelling}")

        ensure_defined = get_message("workflow.ensure_agent_defined")
        lines.append(f"     {ensure_defined}")

        return "\n".join(lines)


class WorkflowTemplateError(WorkflowError):
    """Workflow 模板渲染错误"""

    def __init__(self, template: str, expression: str, context_keys: list = None):
        self.template = template
        self.expression = expression
        self.context_keys = context_keys or []

        message = f"模板渲染失败: 无法解析表达式 '${{{expression}}}'"

        details = {
            "template": template,
            "expression": expression,
            "context_keys": context_keys,
        }

        super().__init__(message, details)

    def __str__(self):
        title = get_message("workflow.template_error_title")
        lines = [title]

        expr_label = get_message("workflow.expression")
        lines.append(f"   {expr_label}: ${{{self.expression}}}")

        # 显示模板片段（最多100字符）
        template_preview = self.template[:100]
        if len(self.template) > 100:
            template_preview += "..."
        snippet_label = get_message("workflow.template_snippet")
        lines.append(f"   {snippet_label}: {template_preview}")

        if self.context_keys:
            keys_label = get_message("workflow.available_context_keys")
            lines.append(f"   {keys_label}:")
            for key in self.context_keys[:10]:  # 最多显示10个
                lines.append(f"     - {key}")

        lines.append("")
        suggestion_label = get_message("workflow.fix_suggestion")
        lines.append(f"   {suggestion_label}:")

        check_syntax = get_message("workflow.check_expression_syntax")
        lines.append(f"     {check_syntax}")

        ensure_defined = get_message("workflow.ensure_variable_defined")
        lines.append(f"     {ensure_defined}")

        check_order = get_message("workflow.check_execution_order")
        lines.append(f"     {check_order}")

        return "\n".join(lines)


class WorkflowAgentDefinitionError(WorkflowError):
    """Workflow agent 定义错误（文件不存在、内容无效等）"""

    def __init__(
        self, agent_id: str, agent_path: str, error_type: str, error_details: str = None
    ):
        self.agent_id = agent_id
        self.agent_path = agent_path
        self.error_type = error_type

        message = f"代理 '{agent_id}' 定义错误: {error_type}"

        details = {
            "agent_id": agent_id,
            "agent_path": agent_path,
            "error_type": error_type,
            "error_details": error_details,
        }

        super().__init__(message, details)

    def __str__(self):
        title = get_message("workflow.agent_definition_error_title")
        lines = [title]

        agent_id_label = get_message("workflow.agent_id")
        lines.append(f"   {agent_id_label}: {self.agent_id}")

        agent_path_label = get_message("workflow.agent_path")
        lines.append(f"   {agent_path_label}: {self.agent_path}")

        error_type_label = get_message("workflow.error_type")
        lines.append(f"   {error_type_label}: {self.error_type}")

        if self.details.get("error_details"):
            error_details_label = get_message("workflow.error_details")
            lines.append(f"   {error_details_label}: {self.details['error_details']}")

        lines.append("")
        suggestion_label = get_message("workflow.fix_suggestion")
        lines.append(f"   {suggestion_label}:")

        check_file = get_message("workflow.check_agent_file_exists")
        lines.append(f"     {check_file}")

        ensure_content = get_message("workflow.ensure_agent_content_valid")
        lines.append(f"     {ensure_content}")

        check_spelling = get_message("workflow.check_path_spelling")
        lines.append(f"     {check_spelling}")

        return "\n".join(lines)


class WorkflowConversationError(WorkflowError):
    """Workflow 会话管理错误"""

    def __init__(
        self,
        message: str,
        conversation_id: str = None,
        action: str = None,
        step_id: str = None,
    ):
        self.conversation_id = conversation_id
        self.action = action
        self.step_id = step_id

        details = {
            "conversation_id": conversation_id,
            "action": action,
            "step_id": step_id,
        }

        super().__init__(message, details)

    def __str__(self):
        title = get_message("workflow.conversation_error_title")
        lines = [f"{title}: {self.message}"]

        if self.step_id:
            step_label = get_message("workflow.step_id")
            lines.append(f"   {step_label}: {self.step_id}")

        if self.action:
            action_label = get_message("workflow.conversation_action")
            lines.append(f"   {action_label}: {self.action}")

        if self.conversation_id:
            conv_id_label = get_message("workflow.conversation_id")
            lines.append(f"   {conv_id_label}: {self.conversation_id}")

        return "\n".join(lines)


class WorkflowConditionError(WorkflowError):
    """Workflow 条件评估错误"""

    def __init__(
        self,
        message: str,
        step_id: str = None,
        condition_type: str = None,
        condition_details: dict = None,
    ):
        self.step_id = step_id
        self.condition_type = condition_type
        self.condition_details = condition_details or {}

        details = {
            "step_id": step_id,
            "condition_type": condition_type,
            **condition_details,
        }

        super().__init__(message, details)

    def __str__(self):
        title = get_message("workflow.condition_error_title")
        lines = [f"{title}: {self.message}"]

        if self.step_id:
            step_label = get_message("workflow.step_id")
            lines.append(f"   {step_label}: {self.step_id}")

        if self.condition_type:
            type_label = get_message("workflow.condition_type")
            lines.append(f"   {type_label}: {self.condition_type}")

        if self.condition_details:
            details_label = get_message("workflow.condition_details")
            lines.append(f"   {details_label}:")
            for key, value in self.condition_details.items():
                lines.append(f"     {key}: {value}")

        lines.append("")
        suggestion_label = get_message("workflow.fix_suggestion")
        lines.append(f"   {suggestion_label}:")

        check_when = get_message("workflow.check_when_syntax")
        lines.append(f"     {check_when}")

        verify_regex = get_message("workflow.verify_regex_pattern")
        lines.append(f"     {verify_regex}")

        ensure_jsonpath = get_message("workflow.ensure_jsonpath_correct")
        lines.append(f"     {ensure_jsonpath}")

        return "\n".join(lines)


class WorkflowOutputExtractionError(WorkflowError):
    """Workflow 输出提取错误"""

    def __init__(
        self,
        message: str,
        step_id: str = None,
        output_key: str = None,
        extraction_method: str = None,
    ):
        self.step_id = step_id
        self.output_key = output_key
        self.extraction_method = extraction_method

        details = {
            "step_id": step_id,
            "output_key": output_key,
            "extraction_method": extraction_method,
        }

        super().__init__(message, details)

    def __str__(self):
        title = get_message("workflow.output_extraction_error_title")
        lines = [f"{title}: {self.message}"]

        if self.step_id:
            step_label = get_message("workflow.step_id")
            lines.append(f"   {step_label}: {self.step_id}")

        if self.output_key:
            key_label = get_message("workflow.output_key")
            lines.append(f"   {key_label}: {self.output_key}")

        if self.extraction_method:
            method_label = get_message("workflow.extraction_method")
            lines.append(f"   {method_label}: {self.extraction_method}")

        lines.append("")
        suggestion_label = get_message("workflow.fix_suggestion")
        lines.append(f"   {suggestion_label}:")

        check_jsonpath = get_message("workflow.check_jsonpath_expression")
        lines.append(f"     {check_jsonpath}")

        verify_regex = get_message("workflow.verify_regex_and_group")
        lines.append(f"     {verify_regex}")

        ensure_format = get_message("workflow.ensure_format_matches")
        lines.append(f"     {ensure_format}")

        return "\n".join(lines)


class WorkflowAgentResolutionError(WorkflowError):
    """Workflow 代理解析错误（代理文件无法解析）"""

    def __init__(
        self,
        agent_id: str,
        path: str,
        searched_paths: list = None,
        field_path: str = None,
        suggestion: str = None,
    ):
        """
        Args:
            agent_id: 代理 ID
            path: 代理路径
            searched_paths: 已搜索的路径列表
            field_path: 字段路径，如 "spec.agents[0].path"
            suggestion: 修复建议
        """
        self.agent_id = agent_id
        self.path = path
        self.searched_paths = searched_paths or []
        self.field_path = field_path
        self.suggestion = suggestion

        message = get_message("workflow.agent_resolution_failed")

        details = {
            "agent_id": agent_id,
            "path": path,
            "searched_paths": searched_paths,
            "field_path": field_path,
            "suggestion": suggestion,
        }

        super().__init__(message, details)

    def __str__(self):
        title = get_message("workflow.agent_resolution_error_title")
        lines = [title]

        agent_id_label = get_message("workflow.agent_id")
        lines.append(f"   {agent_id_label}: {self.agent_id}")

        path_label = get_message("workflow.agent_path")
        lines.append(f"   {path_label}: {self.path}")

        if self.field_path:
            field_label = get_message("workflow.field_path")
            lines.append(f"   {field_label}: {self.field_path}")

        if self.searched_paths:
            searched_label = get_message("workflow.searched_agent_paths")
            lines.append(f"   {searched_label}:")
            for search_path in self.searched_paths:
                lines.append(f"     - {search_path}")

        lines.append("")
        suggestion_label = get_message("workflow.fix_suggestion")
        lines.append(f"   {suggestion_label}:")

        if self.suggestion:
            lines.append(f"     {self.suggestion}")
        else:
            check_exists = get_message("workflow.check_agent_path_exists")
            lines.append(f"     {check_exists}")

            verify_path = get_message("workflow.verify_agent_path_correct")
            lines.append(f"     {verify_path}")

        return "\n".join(lines)


class WorkflowModelValidationError(WorkflowError):
    """Workflow 模型验证错误（模型不存在或密钥未配置）"""

    def __init__(
        self,
        message: str,
        step_id: str = None,
        agent_id: str = None,
        model: str = None,
        suggestion: str = None,
    ):
        """
        Args:
            message: 错误消息
            step_id: 步骤 ID
            agent_id: 代理 ID
            model: 模型名称
            suggestion: 修复建议
        """
        self.step_id = step_id
        self.agent_id = agent_id
        self.model = model
        self.suggestion = suggestion

        details = {
            "step_id": step_id,
            "agent_id": agent_id,
            "model": model,
            "suggestion": suggestion,
        }

        super().__init__(message, details)

    @classmethod
    def for_model_not_found(
        cls, step_id: str, agent_id: str, model: str
    ) -> "WorkflowModelValidationError":
        """创建模型不存在错误"""
        message = get_message("workflow.model_not_found")
        suggestion = get_message("workflow.check_model_registered")
        return cls(
            message=f"{message}: {model}",
            step_id=step_id,
            agent_id=agent_id,
            model=model,
            suggestion=suggestion,
        )

    @classmethod
    def for_key_missing(
        cls, step_id: str, agent_id: str, model: str
    ) -> "WorkflowModelValidationError":
        """创建密钥未配置错误"""
        message = get_message("workflow.model_key_missing")
        suggestion = get_message("workflow.check_api_key_configured")
        return cls(
            message=f"{message}: {model}",
            step_id=step_id,
            agent_id=agent_id,
            model=model,
            suggestion=suggestion,
        )

    def __str__(self):
        title = get_message("workflow.model_validation_error_title")
        lines = [f"{title}: {self.message}"]

        if self.step_id:
            step_label = get_message("workflow.step_id")
            lines.append(f"   {step_label}: {self.step_id}")

        if self.agent_id:
            agent_label = get_message("workflow.agent_id")
            lines.append(f"   {agent_label}: {self.agent_id}")

        if self.model:
            model_label = get_message("workflow.model_name")
            lines.append(f"   {model_label}: {self.model}")

        lines.append("")
        suggestion_label = get_message("workflow.fix_suggestion")
        lines.append(f"   {suggestion_label}:")

        if self.suggestion:
            lines.append(f"     {self.suggestion}")
        else:
            check_registered = get_message("workflow.check_model_registered")
            lines.append(f"     {check_registered}")

            check_key = get_message("workflow.check_api_key_configured")
            lines.append(f"     {check_key}")

            check_spelling = get_message("workflow.check_model_name_spelling")
            lines.append(f"     {check_spelling}")

        return "\n".join(lines)



import os
import difflib
import diff_match_patch as dmp_module
from autocoder.common import AutoCoderArgs, git_utils
from typing import List,Tuple
import pydantic
import byzerllm
from autocoder.common.action_yml_file_manager import ActionYmlFileManager
from autocoder.common.printer import Printer
import hashlib
from pathlib import Path
from autocoder.common.types import CodeGenerateResult, MergeCodeWithoutEffect
from autocoder.common.code_modification_ranker import CodeModificationRanker
from autocoder.common import files as  FileUtils
from autocoder.memory.active_context_manager import ActiveContextManager

class PathAndCode(pydantic.BaseModel):
    path: str
    content: str

def safe_abs_path(res):
    "Gives an abs path, which safely returns a full (not 8.3) windows path"
    res = Path(res).resolve()
    return str(res)

def apply_hunk(content, hunk):
    before, after = hunk_to_before_after(hunk)
    
    # Get line numbers from @@ ... @@ markers
    line_info = hunk[0].split("@@")[1].strip()
    s_line_num = int(line_info.split(" ")[1].lstrip("+"))

    # Split content into lines
    content_lines = content.splitlines()
    
    # Merge changes using difflib
    merged_lines = list(difflib.ndiff(before.splitlines(), after.splitlines()))
    
    # Apply changes to original content
    j = 0
    content_out = content_lines[:s_line_num]
    for line in merged_lines:
        if line.startswith("- "):
            continue
        elif line.startswith("+ "):
            content_out.append(line[2:])
        elif line.startswith("  "):
            if j < len(content_lines):
                content_out.append(content_lines[s_line_num+j])
            j += 1
    
    content_out.extend(content_lines[s_line_num+j:])

    return "\n".join(content_out)


def hunk_to_before_after(hunk, lines=False):
    before = []
    after = []
    op = " "
    for line in hunk:
        if len(line) < 2:
            op = " "
            line = line
        else:
            op = line[0]
            line = line[1:]

        if op == " ":
            before.append(line)
            after.append(line)
        elif op == "-":
            before.append(line)
        elif op == "+":
            after.append(line)

    if lines:
        return before, after

    before = "".join(before)
    after = "".join(after)

    return before, after


class CodeAutoMergeStrictDiff:
    def __init__(self, llm: byzerllm.ByzerLLM, args: AutoCoderArgs):
        self.llm = llm
        self.args = args
        self.printer = Printer()


    def parse_diff_block(self,text: str) -> List[PathAndCode]:
        lines = text.split('\n')
        lines_len = len(lines)    
        start_marker_count = 0       
        inline_start_marker_count = 0 
        block = []
        path_and_code_list = []

        def guard(index):
            return index+1 < lines_len 

        def start_marker(line,index):
            return line.startswith('```diff') and guard(index)
        
        def inline_start_marker(line,index):
            return line.startswith('```') and not line.startswith('```diff') and line.strip() != '```'

        def end_marker(line,index):
            return line.startswith('```') and line.strip() == '```'
        

        for (index,line) in enumerate(lines):
            if start_marker(line,index) and start_marker_count == 0:
                start_marker_count += 1        
            elif (start_marker(line,index) or inline_start_marker(line,index)) and start_marker_count > 0:
                inline_start_marker_count += 1     
                block.append(line)    
            elif end_marker(line,index) and start_marker_count == 1 and inline_start_marker_count == 0:
                start_marker_count -= 1            
                if block:
                    # ori_path = block[0][4:0].strip()                       
                    new_path = block[1][4:].strip()                    
                    content = '\n'.join(block)                                
                    block = []
                    path_and_code_list.append(PathAndCode(path=new_path,content=content))       
            elif end_marker(line,index) and inline_start_marker_count > 0:
                inline_start_marker_count -= 1   
                block.append(line)                 
            elif start_marker_count > 0:
                block.append(line)                

        return path_and_code_list

    def merge_code(self, generate_result: CodeGenerateResult, force_skip_git: bool = False):
        result = self.choose_best_choice(generate_result)
        self._merge_code(result.contents[0], force_skip_git)
        return result

    def choose_best_choice(self, generate_result: CodeGenerateResult) -> CodeGenerateResult:
        if len(generate_result.contents) == 1:
            return generate_result
        
        merge_results = []
        for content,conversations in zip(generate_result.contents,generate_result.conversations):
            merge_result = self._merge_code_without_effect(content)
            merge_results.append(merge_result)

        # If all merge results are None, return first one
        if all(len(result.failed_blocks) != 0 for result in merge_results):
            self.printer.print_in_terminal("all_merge_results_failed")
            return CodeGenerateResult(contents=[generate_result.contents[0]], conversations=[generate_result.conversations[0]])
        
        # If only one merge result is not None, return that one
        not_none_indices = [i for i, result in enumerate(merge_results) if len(result.failed_blocks) == 0]
        if len(not_none_indices) == 1:
            idx = not_none_indices[0]
            self.printer.print_in_terminal("only_one_merge_result_success")
            return CodeGenerateResult(contents=[generate_result.contents[idx]], conversations=[generate_result.conversations[idx]])        

        # 最后，如果有多个，那么根据质量排序再返回
        ranker = CodeModificationRanker(self.llm, self.args)
        ranked_result = ranker.rank_modifications(generate_result,merge_results)        
         
        ## 得到的结果，再做一次合并，第一个通过的返回 , 返回做合并有点重复低效，未来修改。
        for content,conversations in zip(ranked_result.contents,ranked_result.conversations):
            merge_result = self._merge_code_without_effect(content)
            if not merge_result.failed_blocks:
                return CodeGenerateResult(contents=[content], conversations=[conversations])

        # 最后保底，但实际不会出现
        return CodeGenerateResult(contents=[ranked_result.contents[0]], conversations=[ranked_result.conversations[0]])
    

    def abs_root_path(self, path):
        if path.startswith(self.args.source_dir):
            return safe_abs_path(Path(path))
        res = Path(self.args.source_dir) / path
        return safe_abs_path(res)            

    def _merge_code_without_effect(self, content: str) -> MergeCodeWithoutEffect:
        """Merge code without any side effects like git operations or file writing.
        Returns a tuple of:
        - list of (file_path, new_content) tuples for successfully merged blocks
        - list of (file_path, content) tuples for failed to merge blocks"""
        diff_blocks = self.parse_diff_block(content)
        file_content_mapping = {}
        failed_blocks = []
        
        for block in diff_blocks:
            path = block.path
            content = block.content
            full_path = self.abs_root_path(path)
            
            if not os.path.exists(full_path):
                file_content_mapping[full_path] = content
                continue
                
            if full_path not in file_content_mapping:
                file_content_mapping[full_path] = FileUtils.read_file(full_path)
            
            try:
                import patch
                patch_obj = patch.fromstring(content.encode('utf-8'))
                root_path = None
                if not path.startswith(self.args.source_dir):
                    root_path = self.args.source_dir

                # Create a copy of the content to apply patch
                temp_content = file_content_mapping[full_path]
                success = patch_obj.apply(root=root_path, content=temp_content)
                if success:
                    file_content_mapping[full_path] = temp_content
                else:
                    failed_blocks.append((full_path, content))
            except Exception as e:
                self.printer.print_in_terminal("merge_failed", style="yellow", path=full_path, error=str(e))
                failed_blocks.append((full_path, content))
                
        return MergeCodeWithoutEffect(
            success_blocks=[(path, content) for path, content in file_content_mapping.items()],
            failed_blocks=failed_blocks
        )

    def print_diff_blocks(self, diff_blocks: List[PathAndCode]):
        """Print diff blocks for user review using rich library"""
        from rich.syntax import Syntax
        from rich.panel import Panel

        # Group blocks by file path
        file_blocks = {}
        for block in diff_blocks:
            if block.path not in file_blocks:
                file_blocks[block.path] = []
            file_blocks[block.path].append(block.content)

        # Generate formatted text for each file
        formatted_text = ""
        for path, contents in file_blocks.items():
            formatted_text += f"##File: {path}\n"
            for content in contents:
                formatted_text += content + "\n"
            formatted_text += "\n"

        # Print with rich panel
        self.printer.print_in_terminal("diff_blocks_title", style="bold green")
        self.printer.console.print(
            Panel(
                Syntax(formatted_text, "diff", theme="monokai"),
                title="Diff Blocks",
                border_style="green",
                expand=False
            )
        )

    def _merge_code(self, content: str, force_skip_git: bool = False):        
        total = 0
        
        file_content = FileUtils.read_file(self.args.file)
        md5 = hashlib.md5(file_content.encode('utf-8')).hexdigest()
        # get the file name 
        file_name = os.path.basename(self.args.file)
        
        if not force_skip_git and not self.args.skip_commit:
            try:
                git_utils.commit_changes(self.args.source_dir, f"auto_coder_pre_{file_name}_{md5}")
            except Exception as e:            
                self.printer.print_in_terminal("git_init_required", style="red", source_dir=self.args.source_dir, error=str(e))
                return            
       
        diff_blocks = self.parse_diff_block(content)        

        for diff_blocks in diff_blocks:
            path = diff_blocks.path
            content = diff_blocks.content          

            import patch
            patch_obj = patch.fromstring(content.encode('utf-8'))
            root_path = None
            if not path.startswith(self.args.source_dir):
                root_path = self.args.source_dir

            success = patch_obj.apply(root=root_path)
            if not success:
                raise Exception("Error applying diff to file: " + path)
                            
        self.printer.print_in_terminal("files_merged_total", total=total)
        if not force_skip_git and not self.args.skip_commit:
            commit_result = git_utils.commit_changes(
                self.args.source_dir, f"{self.args.query}\nauto_coder_{file_name}"
            )
            action_yml_file_manager = ActionYmlFileManager(self.args.source_dir)
            action_file_name = os.path.basename(self.args.file)
            add_updated_urls = []
            commit_result.changed_files
            for file in commit_result.changed_files:
                add_updated_urls.append(os.path.join(self.args.source_dir, file))

            self.args.add_updated_urls = add_updated_urls
            update_yaml_success = action_yml_file_manager.update_yaml_field(action_file_name, "add_updated_urls", add_updated_urls)
            if not update_yaml_success:                        
                self.printer.print_in_terminal("yaml_save_error", style="red", yaml_file=action_file_name)  
            
            if self.args.enable_active_context:
                active_context_manager = ActiveContextManager(self.llm, self.args.source_dir)
                active_context_manager.process_changes(self.args)
            
            git_utils.print_commit_info(commit_result=commit_result)
        else:
            # Print diff blocks for review
            self.print_diff_blocks(diff_blocks)    

    @byzerllm.prompt(render="jinja2")
    def git_require_msg(self, source_dir: str, error: str) -> str:
        '''
        auto_merge only works for git repositories.
         
        Try to use git init in the source directory. 
        
        ```shell
        cd {{ source_dir }}
        git init .
        ```

        Then try to run auto-coder again.
        Error: {{ error }}
        '''



"""
Test module for ProcessManager functionality.

This module tests the process management functionality including:
- Process creation and configuration
- Process registration and management
- Process cleanup and timeout handling
- Error handling and edge cases
"""

import os
import time
import platform
import threading
from unittest.mock import Mock, patch, MagicMock
import subprocess
import pytest

from autocoder.common.shell_commands.process_manager import ProcessManager
from autocoder.common.shell_commands.timeout_config import TimeoutConfig
from autocoder.common.shell_commands.exceptions import CommandExecutionError, ProcessCleanupError


class TestProcessManager:
    """Test basic ProcessManager functionality."""
    
    def test_initialization(self):
        """Test ProcessManager initialization."""
        config = TimeoutConfig(default_timeout=60.0)
        manager = ProcessManager(config)
        
        assert manager.config == config
        assert manager.timeout_manager is not None
        assert len(manager.active_processes) == 0
        assert len(manager.process_groups) == 0
    
    def test_create_process_basic(self):
        """Test basic process creation."""
        config = TimeoutConfig(default_timeout=60.0)
        manager = ProcessManager(config)
        
        if platform.system() == 'Windows':
            command = ['echo', 'test']
        else:
            command = ['echo', 'test']
        
        process = manager.create_process(command)
        
        try:
            assert process is not None
            assert process.pid > 0
            assert process.pid in manager.active_processes
            
            # Wait for process to complete
            exit_code = manager.wait_for_process(process, timeout=5.0)
            assert exit_code == 0
        finally:
            # Cleanup if needed
            if process.pid in manager.active_processes:
                manager.cleanup_process_tree(process)
    
    def test_create_process_with_options(self):
        """Test process creation with various options."""
        config = TimeoutConfig(default_timeout=60.0)
        manager = ProcessManager(config)
        
        if platform.system() == 'Windows':
            command = ['echo', 'test']
        else:
            command = ['echo', 'test']
        
        process = manager.create_process(
            command,
            timeout=30.0,
            shell=True,
            capture_output=True,
            text=True
        )
        
        try:
            assert process is not None
            assert process.pid > 0
            assert process.pid in manager.active_processes
            
            # Wait for process to complete
            exit_code = manager.wait_for_process(process, timeout=5.0)
            assert exit_code == 0
        finally:
            # Cleanup if needed
            if process.pid in manager.active_processes:
                manager.cleanup_process_tree(process)
    
    def test_create_process_failure(self):
        """Test process creation failure handling."""
        config = TimeoutConfig(default_timeout=60.0)
        manager = ProcessManager(config)
        
        # Try to create process with invalid cwd (this will actually fail)
        with pytest.raises(CommandExecutionError):
            manager.create_process(['echo', 'test'], cwd='/this/path/does/not/exist')
    
    def test_register_process(self):
        """Test manual process registration."""
        config = TimeoutConfig(default_timeout=60.0)
        manager = ProcessManager(config)
        
        mock_process = Mock()
        mock_process.pid = 12345
        
        manager._register_process(mock_process)
        
        assert mock_process.pid in manager.active_processes
    
    def test_cleanup_process_success(self):
        """Test successful process cleanup."""
        config = TimeoutConfig(default_timeout=60.0)
        manager = ProcessManager(config)
        
        if platform.system() == 'Windows':
            command = ['ping', '127.0.0.1', '-n', '1']
        else:
            command = ['sleep', '0.1']
        
        process = manager.create_process(command)
        
        try:
            pid = process.pid
            assert pid in manager.active_processes
            
            # Wait a bit for process to complete or be running
            time.sleep(0.05)
            
            # Cleanup
            result = manager.cleanup_process_tree(process)
            
            assert result is True or result is False  # Should return a boolean
            assert pid not in manager.active_processes
            
        except Exception as e:
            # Fallback cleanup
            try:
                process.terminate()
                process.wait(timeout=1)
            except:
                pass
            manager.unregister_process(process)
    
    def test_cleanup_nonexistent_process(self):
        """Test cleanup of non-existent process."""
        config = TimeoutConfig(default_timeout=60.0)
        manager = ProcessManager(config)
        
        # Create a mock process that doesn't exist
        mock_process = Mock()
        mock_process.pid = 99999
        
        result = manager.cleanup_process_tree(mock_process)
        
        # Should handle gracefully
        assert result is False or result is True
    
    def test_cleanup_all_processes(self):
        """Test cleanup of all managed processes."""
        config = TimeoutConfig(default_timeout=60.0)
        manager = ProcessManager(config)
        
        processes = []
        
        # Create multiple processes
        for i in range(3):
            if platform.system() == 'Windows':
                command = ['ping', '127.0.0.1', '-n', '1']
            else:
                command = ['sleep', '0.1']
            
            process = manager.create_process(command)
            if process:
                processes.append(process)
        
        try:
            assert len(manager.active_processes) >= len(processes)
            
            # Cleanup all
            results = manager.cleanup_all_processes()
            
            assert len(manager.active_processes) == 0
            assert isinstance(results, list)
            
        except Exception:
            # Fallback cleanup
            for process in processes:
                try:
                    process.terminate()
                    process.wait(timeout=1)
                except:
                    pass
            manager.active_processes.clear()
    
    def test_get_managed_processes(self):
        """Test getting list of managed processes."""
        config = TimeoutConfig(default_timeout=60.0)
        manager = ProcessManager(config)
        
        initial_count = len(manager.get_all_processes())
        
        if platform.system() == 'Windows':
            command = ['echo', 'test']
        else:
            command = ['echo', 'test']
        
        process = manager.create_process(command)
        
        try:
            current_count = len(manager.get_all_processes())
            assert current_count == initial_count + 1
            
            processes = manager.get_all_processes()
            assert process.pid in processes
            assert processes[process.pid] == process
            
        finally:
            if process.pid in manager.active_processes:
                manager.cleanup_process_tree(process)
    
    def test_is_process_managed(self):
        """Test checking if process is managed."""
        config = TimeoutConfig(default_timeout=60.0)
        manager = ProcessManager(config)
        
        # Non-existent process
        assert 99999 not in manager.active_processes
        
        if platform.system() == 'Windows':
            command = ['echo', 'test']
        else:
            command = ['echo', 'test']
        
        process = manager.create_process(command)
        
        try:
            # Process should be managed
            assert process.pid in manager.active_processes
            
            # Wait for process to complete
            exit_code = manager.wait_for_process(process, timeout=5.0)
            
            # Process should no longer be managed
            assert process.pid not in manager.active_processes
            
        finally:
            if process.pid in manager.active_processes:
                manager.cleanup_process_tree(process)
    
    def test_unregister_process(self):
        """Test process unregistration."""
        config = TimeoutConfig(default_timeout=60.0)
        manager = ProcessManager(config)
        
        mock_process = Mock()
        mock_process.pid = 12345
        
        # Register then unregister
        manager._register_process(mock_process)
        assert mock_process.pid in manager.active_processes
        
        manager.unregister_process(mock_process)
        assert mock_process.pid not in manager.active_processes


class TestProcessManagerWithTimeout:
    """Test ProcessManager with timeout functionality."""
    
    def test_process_with_timeout(self):
        """Test process creation with timeout."""
        config = TimeoutConfig(default_timeout=1.0)
        manager = ProcessManager(config)
        
        if platform.system() == 'Windows':
            # Use a command that will run for a while
            command = ['ping', '127.0.0.1', '-n', '10']
        else:
            command = ['sleep', '10']
        
        process = manager.create_process(command, timeout=0.1)
        
        try:
            assert process is not None
            
            # Process should be killed by timeout
            start_time = time.time()
            try:
                process.wait(timeout=1.0)
            except subprocess.TimeoutExpired:
                # Expected if process is still running
                pass
            duration = time.time() - start_time
            
            # Should have been terminated by timeout (allow some margin)
            assert duration < 2.0
            
        finally:
            try:
                process.terminate()
                process.wait(timeout=1)
            except:
                pass
            if process.pid in manager.active_processes:
                manager.cleanup_process_tree(process)


class TestProcessManagerErrorHandling:
    """Test ProcessManager error handling."""
    
    def test_cleanup_with_permission_error(self):
        """Test cleanup when permission is denied."""
        config = TimeoutConfig(default_timeout=60.0)
        manager = ProcessManager(config)
        
        # Try to cleanup a system process (should fail with permission error)
        if platform.system() == 'Windows':
            system_pid = 4  # System process
        else:
            system_pid = 1  # Init process
        
        # Create a mock process for system PID
        mock_process = Mock()
        mock_process.pid = system_pid
        
        result = manager.cleanup_process_tree(mock_process)
        
        # Should handle gracefully (might succeed or fail depending on permissions)
        assert result is True or result is False
    
    def test_cleanup_with_process_not_found(self):
        """Test cleanup when process is not found."""
        config = TimeoutConfig(default_timeout=60.0)
        manager = ProcessManager(config)
        
        # Create a mock process that doesn't actually exist
        mock_process = Mock()
        mock_process.pid = 99999
        
        result = manager.cleanup_process_tree(mock_process)
        
        # Should handle gracefully
        assert result is True or result is False
    
    def test_process_creation_with_invalid_command(self):
        """Test process creation with invalid command."""
        config = TimeoutConfig(default_timeout=60.0)
        manager = ProcessManager(config)
        
        # Try to create process with invalid cwd (this will actually fail)
        with pytest.raises(CommandExecutionError):
            manager.create_process(['echo', 'test'], cwd='/this/path/does/not/exist')
    
    def test_process_creation_with_invalid_cwd(self):
        """Test process creation with invalid working directory."""
        config = TimeoutConfig(default_timeout=60.0)
        manager = ProcessManager(config)
        
        with pytest.raises(CommandExecutionError):
            manager.create_process(
                ['echo', 'test'],
                cwd='/this/directory/does/not/exist'
            )
    
    @patch('autocoder.common.shell_commands.process_manager.cleanup_process_tree')
    def test_cleanup_with_cleanup_function_failure(self, mock_cleanup):
        """Test process cleanup when cleanup function fails."""
        config = TimeoutConfig(default_timeout=60.0)
        manager = ProcessManager(config)
        
        # Make cleanup function return False (indicating failure)
        mock_cleanup.return_value = False
        
        # Create a mock process that appears to be still running
        mock_process = Mock()
        mock_process.pid = 123
        mock_process.poll.return_value = None  # Process is still running
        
        # Cleanup should handle the failure
        result = manager.cleanup_process_tree(mock_process)
        
        # Should return False on failure
        assert result is False


class TestProcessManagerConcurrency:
    """Test ProcessManager concurrency handling."""
    
    def test_concurrent_process_creation(self):
        """Test concurrent process creation."""
        import threading
        
        config = TimeoutConfig(default_timeout=60.0)
        manager = ProcessManager(config)
        
        processes = []
        errors = []
        
        def create_process():
            try:
                if platform.system() == 'Windows':
                    command = ['echo', 'test']
                else:
                    command = ['echo', 'test']
                
                process = manager.create_process(command)
                if process:
                    processes.append(process)
            except Exception as e:
                errors.append(e)
        
        # Create multiple threads
        threads = [threading.Thread(target=create_process) for _ in range(5)]
        
        # Start all threads
        for thread in threads:
            thread.start()
        
        # Wait for all threads
        for thread in threads:
            thread.join()
        
        try:
            # Should have created some processes without errors
            assert len(processes) > 0
            assert len(errors) == 0
            
            # All processes should be managed
            for process in processes:
                assert process.pid in manager.active_processes
                
        finally:
            # Cleanup
            for process in processes:
                try:
                    if process.pid in manager.active_processes:
                        manager.cleanup_process_tree(process)
                except:
                    pass


class TestProcessManagerIntegration:
    """Test ProcessManager integration scenarios."""
    
    def test_full_lifecycle(self):
        """Test complete process lifecycle."""
        config = TimeoutConfig(default_timeout=60.0)
        manager = ProcessManager(config)
        
        if platform.system() == 'Windows':
            command = ['echo', 'Hello World']
        else:
            command = ['echo', 'Hello World']
        
        # Create process
        process = manager.create_process(command)
        
        try:
            assert process is not None
            assert process.pid in manager.active_processes
            
            # Wait for completion
            exit_code = manager.wait_for_process(process, timeout=5.0)
            assert exit_code == 0
            
            # Process should be automatically unregistered
            assert process.pid not in manager.active_processes
            
        finally:
            if process.pid in manager.active_processes:
                manager.cleanup_process_tree(process)


class TestProcessManagerEdgeCases:
    """Test ProcessManager edge cases."""
    
    def test_empty_command(self):
        """Test process creation with empty command."""
        config = TimeoutConfig(default_timeout=60.0)
        manager = ProcessManager(config)
        
        # Empty command will create a process but it will fail at runtime
        process = manager.create_process([])
        
        try:
            assert process is not None
            assert process.pid > 0
            
            # Wait for process to fail
            try:
                exit_code = process.wait(timeout=2.0)
                # Process should fail with non-zero exit code
                assert exit_code != 0
            except subprocess.TimeoutExpired:
                # Process might hang, that's also expected behavior
                pass
        finally:
            if process and process.pid in manager.active_processes:
                manager.cleanup_process_tree(process)
    
    def test_very_short_timeout(self):
        """Test process with very short timeout."""
        config = TimeoutConfig(default_timeout=60.0)
        manager = ProcessManager(config)
        
        if platform.system() == 'Windows':
            command = ['ping', '127.0.0.1', '-n', '5']
        else:
            command = ['sleep', '1']
        
        process = manager.create_process(command, timeout=0.001)  # Very short
        
        try:
            if process:
                start_time = time.time()
                try:
                    process.wait(timeout=0.2)
                except subprocess.TimeoutExpired:
                    # Expected if process is still running
                    pass
                duration = time.time() - start_time
                
                # Should have been killed quickly
                assert duration < 0.5
        finally:
            if process and process.pid in manager.active_processes:
                manager.cleanup_process_tree(process)
    
    def test_cleanup_already_cleaned_process(self):
        """Test cleanup of already cleaned process."""
        config = TimeoutConfig(default_timeout=60.0)
        manager = ProcessManager(config)
        
        if platform.system() == 'Windows':
            command = ['echo', 'test']
        else:
            command = ['echo', 'test']
        
        process = manager.create_process(command)
        
        try:
            pid = process.pid
            
            # Wait for process to complete
            process.wait(timeout=5)
            
            # Cleanup once
            result1 = manager.cleanup_process_tree(process)
            
            # Cleanup again
            result2 = manager.cleanup_process_tree(process)
            
            # Both should handle gracefully
            assert result1 is True or result1 is False
            assert result2 is True or result2 is False
            
        except Exception:
            if process and process.pid in manager.active_processes:
                manager.cleanup_process_tree(process)
    
    def test_manager_state_after_errors(self):
        """Test manager state consistency after various errors."""
        config = TimeoutConfig(default_timeout=60.0)
        manager = ProcessManager(config)
        
        initial_count = len(manager.active_processes)
        
        # Try various operations that might fail
        try:
            # This will create a process that fails at runtime
            process1 = manager.create_process(['this_command_does_not_exist'])
            if process1:
                manager.cleanup_process_tree(process1)
        except CommandExecutionError:
            pass
        
        try:
            manager.create_process(['echo', 'test'], cwd='/invalid/path')
        except CommandExecutionError:
            pass
        
        # Manager state should be consistent (might have processes that were created)
        current_count = len(manager.active_processes)
        
        # Should still be able to create valid processes
        if platform.system() == 'Windows':
            command = ['echo', 'test']
        else:
            command = ['echo', 'test']
        
        process = manager.create_process(command)
        
        try:
            assert process is not None
            assert len(manager.active_processes) >= current_count
            
        finally:
            if process and process.pid in manager.active_processes:
                manager.cleanup_process_tree(process)


if __name__ == "__main__":
    pytest.main([__file__, "-v"]) 


import locale
from byzerllm.utils import format_str_jinja2

MESSAGES = {
    "rag_error_title": {
        "en": "RAG Error",
        "zh": "RAG 错误"
    },
    "rag_error_message": {
        "en": "Failed to generate response: {{error}}",
        "zh": "生成响应失败: {{error}}"
    },
    "rag_searching_docs": {
        "en": "Searching documents with {{model}}...",
        "zh": "正在使用 {{model}} 搜索文档..."
    },
    "rag_docs_filter_result": {
        "en": "{{model}} processed {{docs_num}} documents, cost {{filter_time}} seconds, input tokens: {{input_tokens}}, output tokens: {{output_tokens}}",
        "zh": "{{model}} 处理了 {{docs_num}} 个文档, 耗时 {{filter_time}} 秒, 输入 tokens: {{input_tokens}}, 输出 tokens: {{output_tokens}}"
    },
    "dynamic_chunking_start": {
        "en": "Dynamic chunking start with {{model}}",
        "zh": "使用 {{model}} 进行动态分块"
    },
    "dynamic_chunking_result": {
        "en": "Dynamic chunking result with {{model}}, first round cost {{first_round_time}} seconds, second round cost {{sencond_round_time}} seconds, input tokens: {{input_tokens}}, output tokens: {{output_tokens}}, first round full docs: {{first_round_full_docs}}, second round extracted docs: {{second_round_extracted_docs}}",
        "zh": "使用 {{model}} 进行动态分块, 第一轮耗时 {{first_round_time}} 秒, 第二轮耗时 {{sencond_round_time}} 秒, 输入 tokens: {{input_tokens}}, 输出 tokens: {{output_tokens}}, 第一轮全量文档: {{first_round_full_docs}}, 第二轮提取文档: {{second_round_extracted_docs}}"
    },
    "send_to_model": {
        "en": "Send to model {{model}} with {{tokens}} tokens",
        "zh": "发送给模型 {{model}} 的 tokens 数量预估为 {{tokens}}"
    },
    "doc_filter_start": {
        "en": "Document filtering start, total {{total}} documents",
        "zh": "开始过滤文档，共 {{total}} 个文档"
    },
    "doc_filter_progress": {
        "en": "Document filtering progress: {{progress_percent}}% processed {{relevant_count}}/{{total}} documents",
        "zh": "文档过滤进度：{{progress_percent}}%，处理了 {{relevant_count}}/{{total}} 个文档"
    },
    "doc_filter_error": {
        "en": "Document filtering error: {{error}}",
        "zh": "文档过滤错误：{{error}}"
    },
    "doc_filter_complete": {
        "en": "Document filtering complete, cost {{total_time}} seconds, found {{relevant_count}} relevant documents",
        "zh": "文档过滤完成，耗时 {{total_time}} 秒，找到 {{relevant_count}} 个相关文档"
    },
    "context_docs_names": {
        "en": "The following are the documents related to the user's question: {{context_docs_names}}",
        "zh": "以下是和用户问题相关的文档：{{context_docs_names}}"
    },
    "rag_processing": {
        "en": "Processing...\n",
        "zh": "处理中...\n"
    }
}


def get_system_language():
    try:
        return locale.getdefaultlocale()[0][:2]
    except:
        return 'en'


def get_message(key):
    lang = get_system_language()
    if key in MESSAGES:
        return MESSAGES[key].get(lang, MESSAGES[key].get("en", ""))
    return ""


def get_message_with_format(msg_key: str, **kwargs):
    return format_str_jinja2(get_message(msg_key), **kwargs)

def get_message_with_format_and_newline(msg_key: str, **kwargs):
    return format_str_jinja2(get_message(msg_key), **kwargs) + "\n"




import pytest
from collections import OrderedDict
from typing import Dict, List, Any

# 导入被测模块
from .parser import (
    CommandParser,
    parse_query,
    has_command,
    get_command_args,
    get_command_kwargs,
    get_query_command
)


class TestCommandParser:
    """CommandParser类的单元测试"""
    
    @pytest.fixture
    def parser(self):
        """提供CommandParser实例"""
        return CommandParser()
    
    def test_init(self, parser):
        """测试CommandParser初始化"""
        assert parser is not None
        assert hasattr(parser, 'command_pattern')
        assert hasattr(parser, 'key_value_pattern') 
        assert hasattr(parser, 'path_pattern')
    
    def test_empty_query(self, parser):
        """测试空查询字符串"""
        assert parser.parse("") == {}
        assert parser.parse("   ") == {}
        assert parser.parse(None) == {}
    
    def test_single_command_no_args(self, parser):
        """测试单个命令无参数"""
        result = parser.parse("/help")
        expected = {
            "help": {
                "args": [],
                "kwargs": {}
            }
        }
        assert result == expected
    
    def test_single_command_with_args(self, parser):
        """测试单个命令带位置参数"""
        result = parser.parse("/add file1.txt file2.py")
        expected = {
            "add": {
                "args": ["file1.txt", "file2.py"],
                "kwargs": {}
            }
        }
        assert result == expected
    
    def test_single_command_with_kwargs(self, parser):
        """测试单个命令带键值对参数"""
        result = parser.parse("/config model=gpt-4 temperature=0.7")
        expected = {
            "config": {
                "args": [],
                "kwargs": {
                    "model": "gpt-4",
                    "temperature": "0.7"
                }
            }
        }
        assert result == expected
    
    def test_single_command_mixed_params(self, parser):
        """测试单个命令混合参数"""
        result = parser.parse("/deploy myapp env=prod version=1.0")
        expected = {
            "deploy": {
                "args": ["myapp"],
                "kwargs": {
                    "env": "prod",
                    "version": "1.0"
                }
            }
        }
        assert result == expected

    def test_single_command_new_params(self, parser):
        """测试单个命令混合参数"""
        result = parser.parse('/new "把 /Users/williamzhu/.auto-coder/async_agent/tasks/stdin_20250815101939_e9042b6c 中未commit变更合并回来"')
        expected = {
            "new": {
                "args": ['把 /Users/williamzhu/.auto-coder/async_agent/tasks/stdin_20250815101939_e9042b6c 中未commit变更合并回来'],
                "kwargs": {}
            }
        }
        assert result == expected   

    def test_single_command_async_params(self, parser):
        """测试单个命令混合参数"""
        result = parser.parse("/async \"/list @./src/agent.ts 中,while 循环里有loop call limit 限制,当前会发出一个 emitEvent 事件,同时也需要发出一个渲染事件(opts.render方法)\"")
        expected = {
            "async": {
                "args": ['/list @./src/agent.ts 中,while 循环里有loop call limit 限制,当前会发出一个 emitEvent 事件,同时也需要发出一个渲染事件(opts.render方法)'],
                "kwargs": {}
            }
        }
        assert result == expected      
    
    def test_quoted_args_double_quotes(self, parser):
        """测试双引号参数"""
        result = parser.parse('/say "hello world" "this is a test"')
        expected = {
            "say": {
                "args": ["hello world", "this is a test"],
                "kwargs": {}
            }
        }
        assert result == expected
    
    def test_quoted_args_single_quotes(self, parser):
        """测试单引号参数"""
        result = parser.parse("/say 'hello world' 'this is a test'")
        expected = {
            "say": {
                "args": ["hello world", "this is a test"],
                "kwargs": {}
            }
        }
        assert result == expected
    
    def test_quoted_kwargs(self, parser):
        """测试带引号的键值对参数"""
        result = parser.parse('/config message="hello world" path=\'/tmp/test dir\'')
        expected = {
            "config": {
                "args": [],
                "kwargs": {
                    "message": "hello world",
                    "path": "/tmp/test dir"
                }
            }
        }
        assert result == expected
    
    def test_multiple_commands(self, parser):
        """测试多个命令"""
        result = parser.parse("/add file1.txt /remove file2.txt")
        expected = {
            "add": {
                "args": ["file1.txt"],
                "kwargs": {}
            },
            "remove": {
                "args": ["file2.txt"],
                "kwargs": {}
            }
        }
        assert result == expected
    
    def test_multiple_commands_with_mixed_params(self, parser):
        """测试多个命令带混合参数"""
        result = parser.parse("/deploy app1 env=prod /config model=gpt-4 /status")
        expected = {
            "deploy": {
                "args": ["app1"],
                "kwargs": {"env": "prod"}
            },
            "config": {
                "args": [],
                "kwargs": {"model": "gpt-4"}
            },
            "status": {
                "args": [],
                "kwargs": {}
            }
        }
        assert result == expected
    
    def test_command_order_preservation(self, parser):
        """测试命令顺序保持"""
        result = parser.parse("/third /first /second arg1 /fourth key=value")
        
        # 验证返回的是OrderedDict
        assert isinstance(result, OrderedDict)
        
        # 验证命令顺序
        command_order = list(result.keys())
        expected_order = ["third", "first", "second", "fourth"]
        assert command_order == expected_order
        
        # 验证内容正确性
        assert result["third"]["args"] == []
        assert result["first"]["args"] == []
        assert result["second"]["args"] == ["arg1"]
        assert result["fourth"]["kwargs"] == {"key": "value"}
    
    def test_path_not_recognized_as_command(self, parser):
        """测试路径不被识别为命令"""
        result = parser.parse("/add /path/to/file.txt /config model=gpt-4")
        expected = {
            "add": {
                "args": ["/path/to/file.txt"],
                "kwargs": {}
            },
            "config": {
                "args": [],
                "kwargs": {"model": "gpt-4"}
            }
        }
        assert result == expected
    
    def test_complex_path_handling(self, parser):
        """测试复杂路径处理"""
        result = parser.parse("/analyze /home/user/project/src/main.py /output /tmp/results")
        expected = {
            "analyze": {
                "args": ["/home/user/project/src/main.py"],
                "kwargs": {}
            },
            "output": {
                "args": ["/tmp/results"],
                "kwargs": {}
            }
        }
        assert result == expected
    
    def test_command_with_dots_not_recognized(self, parser):
        """测试带点的字符串不被识别为命令"""
        result = parser.parse("check /config.json /setup.py")
        # 不以/开头，所以没有命令被识别
        assert result == {}
    
    def test_parse_command_existing(self, parser):
        """测试解析特定存在的命令"""
        query = "/add file1.txt /remove file2.txt mode=force"
        result = parser.parse_command(query, "remove")
        expected = {
            "args": ["file2.txt"],
            "kwargs": {"mode": "force"}
        }
        assert result == expected
    
    def test_parse_command_nonexistent(self, parser):
        """测试解析不存在的命令"""
        query = "/add file1.txt /remove file2.txt"
        result = parser.parse_command(query, "deploy")
        assert result is None
    
    def test_edge_case_empty_kwargs_value(self, parser):
        """测试空的键值对值"""
        result = parser.parse("/config key1= key2=value")
        expected = {
            "config": {
                "args": [],
                "kwargs": {
                    "key1": "",
                    "key2": "value"
                }
            }
        }
        assert result == expected
    
    def test_edge_case_special_characters_in_values(self, parser):
        """测试值中包含特殊字符"""
        result = parser.parse("/config url=https://api.example.com/v1 pattern='*.py'")
        expected = {
            "config": {
                "args": [],
                "kwargs": {
                    "url": "https://api.example.com/v1",
                    "pattern": "*.py"
                }
            }
        }
        assert result == expected
    
    def test_command_at_end_of_string(self, parser):
        """测试字符串末尾的命令"""
        result = parser.parse("some text /help")
        expected = {
            "help": {
                "args": [],
                "kwargs": {}
            }
        }
        assert result == expected
    
    def test_auto_new_complex_scenario(self, parser):
        """测试复杂的自动化场景：/auto /new 运行使用交互式方式运行 auto-coder.chat 然后在里面运行 /rules /list"""
        query = "/auto /new 运行使用交互式方式运行 auto-coder.chat 然后在里面运行 /rules /list"
        result = parser.parse(query)
        expected = {
            "auto": {
                "args": [],
                "kwargs": {}
            },
            "new": {
                "args": ["运行使用交互式方式运行", "auto-coder.chat", "然后在里面运行"],
                "kwargs": {}
            },
            "rules": {
                "args": [],
                "kwargs": {}
            },
            "list": {
                "args": [],
                "kwargs": {}
            }
        }
        assert result == expected

    def test_auto_new_complex_scenario2(self, parser):
        """测试复杂的自动化场景：/auto /new 运行使用交互式方式运行 auto-coder.chat 然后在里面运行 /rules /list"""
        query = '/auto /new "运行使用交互式方式运行 auto-coder.chat 然后在里面运行 /rules /list"'
        result = parser.parse(query)
        expected = {
            "auto": {
                "args": [],
                "kwargs": {}
            },
            "new": {
                "args": ["运行使用交互式方式运行 auto-coder.chat 然后在里面运行 /rules /list"],
                "kwargs": {}
            }            
        }
        assert result == expected    


class TestConvenienceFunctions:
    """测试便捷函数"""
    
    def test_parse_query_function(self):
        """测试parse_query函数"""
        result = parse_query("/add file1.txt mode=fast")
        
        # 验证返回的是OrderedDict
        assert isinstance(result, OrderedDict)
        
        expected = {
            "add": {
                "args": ["file1.txt"],
                "kwargs": {"mode": "fast"}
            }
        }
        assert result == expected
    
    def test_has_command_true(self):
        """测试has_command函数 - 命令存在"""
        assert has_command("/add file1.txt /remove file2.txt", "add") == True
        assert has_command("/add file1.txt /remove file2.txt", "remove") == True
    
    def test_has_command_false(self):
        """测试has_command函数 - 命令不存在"""
        assert has_command("/add file1.txt /remove file2.txt", "deploy") == False
        assert has_command("no commands here", "add") == False
        assert has_command("", "add") == False
    
    def test_get_command_args_existing(self):
        """测试get_command_args函数 - 命令存在"""
        query = "/add file1.txt file2.py /remove file3.txt"
        assert get_command_args(query, "add") == ["file1.txt", "file2.py"]
        assert get_command_args(query, "remove") == ["file3.txt"]
    
    def test_get_command_args_nonexistent(self):
        """测试get_command_args函数 - 命令不存在"""
        query = "/add file1.txt file2.py"
        assert get_command_args(query, "remove") == []
        assert get_command_args("", "add") == []
    
    def test_get_command_kwargs_existing(self):
        """测试get_command_kwargs函数 - 命令存在"""
        query = "/config model=gpt-4 temp=0.7 /deploy env=prod"
        assert get_command_kwargs(query, "config") == {"model": "gpt-4", "temp": "0.7"}
        assert get_command_kwargs(query, "deploy") == {"env": "prod"}
    
    def test_get_command_kwargs_nonexistent(self):
        """测试get_command_kwargs函数 - 命令不存在"""
        query = "/config model=gpt-4"
        assert get_command_kwargs(query, "deploy") == {}
        assert get_command_kwargs("", "config") == {}


class TestComplexScenarios:
    """测试复杂场景"""
    
    def test_real_world_scenario_1(self):
        """测试真实场景1：文件操作命令"""
        query = '/add "src/main.py" "tests/test_main.py" /config model="gpt-4" temperature=0.7 /deploy env=production version="1.2.3"'
        result = parse_query(query)
        
        expected = {
            "add": {
                "args": ["src/main.py", "tests/test_main.py"],
                "kwargs": {}
            },
            "config": {
                "args": [],
                "kwargs": {
                    "model": "gpt-4",
                    "temperature": "0.7"
                }
            },
            "deploy": {
                "args": [],
                "kwargs": {
                    "env": "production",
                    "version": "1.2.3"
                }
            }
        }
        assert result == expected
    
    def test_real_world_scenario_2(self):
        """测试真实场景2：包含路径的复杂命令"""
        query = "/analyze /home/user/project/src /output /tmp/analysis.json format=json verbose=true"
        result = parse_query(query)
        
        expected = {
            "analyze": {
                "args": ["/home/user/project/src"],
                "kwargs": {}
            },
            "output": {
                "args": ["/tmp/analysis.json"],
                "kwargs": {
                    "format": "json",
                    "verbose": "true"
                }
            }
        }
        assert result == expected
    
    def test_real_world_scenario_3(self):
        """测试真实场景3：混合引号和特殊字符"""
        query = """/search pattern="*.py" path='/home/user/My Documents/project' /filter exclude="__pycache__" """
        result = parse_query(query)
        
        expected = {
            "search": {
                "args": [],
                "kwargs": {
                    "pattern": "*.py",
                    "path": "/home/user/My Documents/project"
                }
            },
            "filter": {
                "args": [],
                "kwargs": {
                    "exclude": "__pycache__"
                }
            }
        }
        assert result == expected
    
    def test_edge_case_consecutive_commands(self):
        """测试连续命令"""
        query = "/start /stop /restart mode=fast"
        result = parse_query(query)
        
        expected = {
            "start": {
                "args": [],
                "kwargs": {}
            },
            "stop": {
                "args": [],
                "kwargs": {}
            },
            "restart": {
                "args": [],
                "kwargs": {"mode": "fast"}
            }
        }
        assert result == expected
    
    def test_edge_case_command_with_numbers_and_underscores(self):
        """测试包含数字和下划线的命令"""
        query = "/test_case_1 /deploy_v2 app_name=test123"
        result = parse_query(query)
        
        expected = {
            "test_case_1": {
                "args": [],
                "kwargs": {}
            },
            "deploy_v2": {
                "args": [],
                "kwargs": {"app_name": "test123"}
            }
        }
        assert result == expected


class TestErrorHandling:
    """测试错误处理和边界情况"""
    
    def test_malformed_quotes(self):
        """测试格式错误的引号"""
        # 这些应该仍能部分解析，不会抛出异常
        parser = CommandParser()
        
        # 未闭合的引号 - 应该被当作普通参数处理
        result = parser.parse('/test "unclosed quote arg')
        assert "test" in result
        
        # 混合引号
        result = parser.parse('/test "mixed\' quotes"')
        assert "test" in result
    
    def test_special_characters_in_commands(self):
        """测试命令中的特殊字符"""
        parser = CommandParser()
        
        # 命令名可以包含单词字符和连字符，但不能包含点号
        result = parser.parse("/test-command /test.command")
        expected = OrderedDict({
            'test-command': {'args': ['/test.command'], 'kwargs': {}}
        })
        assert result == expected
    
    def test_whitespace_handling(self):
        """测试空白字符处理"""
        parser = CommandParser()
        
        query = "  /add    file1.txt    file2.py   key=value   "
        result = parser.parse(query)
        
        expected = {
            "add": {
                "args": ["file1.txt", "file2.py"],
                "kwargs": {"key": "value"}
            }
        }
        assert result == expected
    
    def test_unicode_characters(self):
        """测试Unicode字符"""
        parser = CommandParser()
        
        query = '/test "中文参数" key="中文值"'
        result = parser.parse(query)
        
        expected = {
            "test": {
                "args": ["中文参数"],
                "kwargs": {"key": "中文值"}
            }
        }
        assert result == expected


# 参数化测试用例
@pytest.mark.parametrize("query,expected_commands", [
    ("/help", ["help"]),
    ("/add /remove", ["add", "remove"]),
    ("/config model=gpt-4", ["config"]),
    ("no commands", []),
    ("/start /stop /restart", ["start", "stop", "restart"]),
])
def test_command_detection_parametrized(query, expected_commands):
    """参数化测试命令检测"""
    result = parse_query(query)
    actual_commands = list(result.keys())
    assert actual_commands == expected_commands


@pytest.mark.parametrize("query,command,expected_args", [
    ("/add file1 file2", "add", ["file1", "file2"]),
    ("/remove", "remove", []),
    ("/test arg1 'arg with spaces'", "test", ["arg1", "arg with spaces"]),
    ("", "any", []),
])
def test_get_args_parametrized(query, command, expected_args):
    """参数化测试获取参数"""
    assert get_command_args(query, command) == expected_args


@pytest.mark.parametrize("query,command,expected_kwargs", [
    ("/config model=gpt-4 temp=0.7", "config", {"model": "gpt-4", "temp": "0.7"}),
    ("/deploy", "deploy", {}),
    ("/test key='value with spaces'", "test", {"key": "value with spaces"}),
    ("", "any", {}),
])
def test_get_kwargs_parametrized(query, command, expected_kwargs):
    """参数化测试获取键值对参数"""
    assert get_command_kwargs(query, command) == expected_kwargs


class TestQueryCommand:
    """测试 QueryCommand 类和 get_query_command 函数"""
    
    def test_basic_query_command(self):
        """测试基本的 QueryCommand 功能"""
        query = "/model gpt-4 /temperature 0.7"
        v = get_query_command(query)
        
        assert v.model == "gpt-4"
        assert v.temperature == "0.7"
        assert v.query == ""
    
    def test_query_command_with_sub_commands(self):
        """测试带子命令的 QueryCommand"""
        query = "/async /model gpt-4 /task-prefix mytask hello world"
        v = get_query_command(query, sub_commands=["model", "task-prefix"])
        
        assert v.model == "gpt-4"
        assert v.task_prefix == "mytask"
        assert v.query == "hello world"
    
    def test_query_command_hyphen_to_underscore(self):
        """测试连字符自动转换为下划线"""
        query = "/task-prefix mytask /api-key secret123"
        v = get_query_command(query)
        
        # 可以用下划线访问
        assert v.task_prefix == "mytask"
        assert v.api_key == "secret123"
    
    def test_query_command_multiple_args(self):
        """测试多个参数的处理"""
        query = "/async /model gpt-4 /task-prefix mytask arg1 arg2 arg3"
        v = get_query_command(query, sub_commands=["model", "task-prefix"])
        
        assert v.model == "gpt-4"
        assert v.task_prefix == "mytask"
        assert v.query == "arg1 arg2 arg3"
    
    def test_query_command_no_special_handling(self):
        """测试不进行特殊处理的情况"""
        query = "/model gpt-4 extra /task-prefix mytask arg1 arg2"
        v = get_query_command(query, sub_commands=None)
        
        # 没有指定sub_commands，所以不会特殊处理
        assert v.model == ["gpt-4", "extra"]
        assert v.task_prefix == ["mytask", "arg1", "arg2"]
        assert v.query == ""
    
    def test_query_command_with_kwargs(self):
        """测试带键值对参数的命令"""
        query = "/config model=gpt-4 temperature=0.7"
        v = get_query_command(query)
        
        config = v.config
        assert isinstance(config, dict)
        assert config["model"] == "gpt-4"
        assert config["temperature"] == "0.7"
    
    def test_query_command_mixed_args_kwargs(self):
        """测试混合参数的命令"""
        query = "/deploy myapp env=prod version=1.0"
        v = get_query_command(query)
        
        # deploy 命令有位置参数和键值对参数
        # 当有kwargs时，返回kwargs
        deploy_info = v.get_command_info("deploy")
        assert deploy_info["args"] == ["myapp"]
        assert deploy_info["kwargs"] == {"env": "prod", "version": "1.0"}
    
    def test_query_command_has_command(self):
        """测试 has_command 方法"""
        query = "/model gpt-4 /task-prefix mytask"
        v = get_query_command(query)
        
        assert v.has_command("model") == True
        assert v.has_command("task-prefix") == True
        assert v.has_command("task_prefix") == True  # 也可以用下划线形式
        assert v.has_command("nonexistent") == False
    
    def test_query_command_get_all_commands(self):
        """测试 get_all_commands 方法"""
        query = "/first /second /third"
        v = get_query_command(query)
        
        commands = v.get_all_commands()
        assert commands == ["first", "second", "third"]
    
    def test_query_command_nonexistent_attribute(self):
        """测试访问不存在的属性"""
        query = "/model gpt-4"
        v = get_query_command(query)
        
        assert v.model == "gpt-4"
        assert v.nonexistent is None
    
    def test_query_command_empty_command(self):
        """测试空参数的命令"""
        query = "/start /stop /restart"
        v = get_query_command(query)
        
        assert v.start == ""
        assert v.stop == ""
        assert v.restart == ""
    
    def test_query_command_complex_scenario(self):
        """测试复杂场景"""
        query = '/async /model gpt-4-turbo /task-prefix analysis "Process the following data" with multiple words'
        v = get_query_command(query, sub_commands=["model", "task-prefix"])
        
        assert v.model == "gpt-4-turbo"
        assert v.task_prefix == "analysis"  # 第一个参数
        assert v.query == 'Process the following data with multiple words'  # 剩余参数
    
    def test_query_command_last_command_not_in_list(self):
        """测试最后一个命令不在子命令列表中"""
        query = "/model gpt-4 /other command /final arg1 arg2 arg3"
        v = get_query_command(query, sub_commands=["model"])
        
        # model 是子命令但不是最后一个，所以保持原样
        assert v.model == "gpt-4"
        # other 和 final 不在子命令列表中，保持原样
        assert v.other == "command"
        assert v.final == ["arg1", "arg2", "arg3"]
        assert v.query == ""
    
    def test_query_command_disable_last_takes_rest(self):
        """测试禁用 last_command_takes_rest"""
        query = "/model gpt-4 /task-prefix mytask arg1 arg2"
        v = get_query_command(query, 
                            sub_commands=["model", "task-prefix"],
                            last_command_takes_rest=False)
        
        # 禁用了特殊处理，所以所有参数都保留
        assert v.model == "gpt-4"
        assert v.task_prefix == ["mytask", "arg1", "arg2"]
        assert v.query == ""
    
    def test_query_command_repr(self):
        """测试 __repr__ 方法"""
        query = "/model gpt-4 /task-prefix mytask hello"
        v = get_query_command(query, sub_commands=["task-prefix"])
        
        repr_str = repr(v)
        assert "QueryCommand" in repr_str
        assert "model" in repr_str
        assert "task-prefix" in repr_str


if __name__ == "__main__":
    # 可以直接运行此文件进行测试
    pytest.main([__file__, "-v"])



"""
文件存储实现

基于JSON文件的对话存储实现，支持原子写入和数据完整性检查。
"""

import os
import json
import tempfile
import re
from typing import Optional, List, Dict, Any
from pathlib import Path

from .base_storage import BaseStorage
from ..exceptions import DataIntegrityError


class FileStorage(BaseStorage):
    """基于文件的存储实现"""
    
    def __init__(self, storage_path: str):
        """
        初始化文件存储
        
        Args:
            storage_path: 存储目录路径
        """
        self.storage_path = Path(storage_path)
        self._ensure_storage_directory()
    
    def _ensure_storage_directory(self):
        """确保存储目录存在"""
        self.storage_path.mkdir(parents=True, exist_ok=True)
    
    def _get_conversation_file_path(self, conversation_id: str) -> Path:
        """
        获取对话文件路径
        
        Args:
            conversation_id: 对话ID
            
        Returns:
            Path: 对话文件路径
        """
        # 清理文件名中的特殊字符
        safe_filename = self._sanitize_filename(conversation_id)
        return self.storage_path / f"{safe_filename}.json"
    
    def _sanitize_filename(self, filename: str) -> str:
        """
        清理文件名，移除或替换特殊字符
        
        Args:
            filename: 原始文件名
            
        Returns:
            str: 安全的文件名
        """
        # 移除或替换不安全的字符
        safe_filename = re.sub(r'[<>:"/\\|?*]', '_', filename)
        # 确保文件名不为空
        if not safe_filename or safe_filename.isspace():
            safe_filename = 'unnamed'
        return safe_filename
    
    def _validate_conversation_data(self, conversation_data: Dict[str, Any]) -> bool:
        """
        验证对话数据的完整性
        
        Args:
            conversation_data: 对话数据
            
        Returns:
            bool: 数据有效返回True
        """
        if not isinstance(conversation_data, dict):
            return False
        
        # 检查必需字段
        required_fields = ['conversation_id']
        for field in required_fields:
            if field not in conversation_data:
                return False
            if not conversation_data[field]:
                return False
        
        return True
    
    def _atomic_write_file(self, file_path: Path, data: Dict[str, Any]) -> bool:
        """
        原子写入文件
        
        Args:
            file_path: 目标文件路径
            data: 要写入的数据
            
        Returns:
            bool: 写入成功返回True
        """
        temp_fd = None
        temp_path = None
        
        try:
            # 创建临时文件
            temp_fd, temp_path = tempfile.mkstemp(
                suffix='.tmp',
                prefix=file_path.name + '.',
                dir=file_path.parent
            )
            
            # 写入数据到临时文件
            with os.fdopen(temp_fd, 'w', encoding='utf-8') as temp_file:
                temp_fd = None  # 文件已关闭，避免重复关闭
                json.dump(data, temp_file, ensure_ascii=False, indent=2)
            
            # 原子重命名（使用 os.replace 以兼容 Windows，os.rename 在 Windows 上目标存在时会报错）
            os.replace(temp_path, file_path)
            return True
            
        except (OSError, IOError, PermissionError, TypeError, ValueError):
            # 清理临时文件
            if temp_fd is not None:
                try:
                    os.close(temp_fd)
                except OSError:
                    pass
            
            if temp_path and os.path.exists(temp_path):
                try:
                    os.unlink(temp_path)
                except OSError:
                    pass
            
            return False
    
    def save_conversation(self, conversation_data: Dict[str, Any]) -> bool:
        """
        保存对话数据
        
        Args:
            conversation_data: 对话数据字典
            
        Returns:
            bool: 保存成功返回True
        """
        if not self._validate_conversation_data(conversation_data):
            return False
        
        conversation_id = conversation_data['conversation_id']
        file_path = self._get_conversation_file_path(conversation_id)
        
        return self._atomic_write_file(file_path, conversation_data)
    
    def load_conversation(self, conversation_id: str) -> Optional[Dict[str, Any]]:
        """
        加载对话数据
        
        Args:
            conversation_id: 对话ID
            
        Returns:
            Optional[Dict[str, Any]]: 对话数据，不存在返回None
        """
        file_path = self._get_conversation_file_path(conversation_id)
        
        if not file_path.exists():
            return None
        
        try:
            with open(file_path, 'r', encoding='utf-8') as f:
                data = json.load(f)
            
            # 验证加载的数据
            if not self._validate_conversation_data(data):
                raise DataIntegrityError(f"对话数据无效: {conversation_id}")
            
            return data
            
        except (json.JSONDecodeError, UnicodeDecodeError) as e:
            raise DataIntegrityError(f"对话文件损坏: {conversation_id}, 错误: {str(e)}")
        except (OSError, IOError) as e:
            # 文件读取错误，返回None
            return None
    
    def delete_conversation(self, conversation_id: str) -> bool:
        """
        删除对话数据
        
        Args:
            conversation_id: 对话ID
            
        Returns:
            bool: 删除成功返回True
        """
        file_path = self._get_conversation_file_path(conversation_id)
        
        if not file_path.exists():
            return False
        
        try:
            file_path.unlink()
            return True
        except (OSError, IOError):
            return False
    
    def conversation_exists(self, conversation_id: str) -> bool:
        """
        检查对话是否存在
        
        Args:
            conversation_id: 对话ID
            
        Returns:
            bool: 存在返回True
        """
        file_path = self._get_conversation_file_path(conversation_id)
        return file_path.exists()
    
    def list_conversations(
        self, 
        limit: Optional[int] = None, 
        offset: int = 0
    ) -> List[Dict[str, Any]]:
        """
        列出对话
        
        Args:
            limit: 限制返回数量
            offset: 偏移量
            
        Returns:
            List[Dict[str, Any]]: 对话数据列表
        """
        conversations = []
        
        try:
            # 获取所有JSON文件
            json_files = list(self.storage_path.glob("*.json"))
            
            # 按修改时间排序（最新的在前）
            json_files.sort(key=lambda x: x.stat().st_mtime, reverse=True)
            
            # 应用偏移量和限制
            if limit is not None:
                json_files = json_files[offset:offset + limit]
            else:
                json_files = json_files[offset:]
            
            # 加载对话数据
            for file_path in json_files:
                try:
                    with open(file_path, 'r', encoding='utf-8') as f:
                        data = json.load(f)
                    
                    # 验证数据
                    if self._validate_conversation_data(data):
                        conversations.append(data)
                        
                except (json.JSONDecodeError, UnicodeDecodeError, OSError, IOError):
                    # 跳过损坏的文件
                    continue
                    
        except OSError:
            # 目录访问错误，返回空列表
            pass
        
        return conversations 


from typing import List, Dict, Any, Optional,Union
from autocoder.common import AutoCoderArgs, SourceCode
from byzerllm import ByzerLLM
from .long_context_rag import LongContextRAG
class RAGFactory:
    
    
    @staticmethod
    def get_rag(llm: ByzerLLM, args: AutoCoderArgs, path: str,**kargs) -> Union[LongContextRAG]:
        """
        Factory method to get the appropriate RAG implementation based on arguments.
        
        Args:
            llm (ByzerLLM): The ByzerLLM instance.
            args (AutoCoderArgs): The arguments for configuring RAG.
            path (str): The path to the data.
            
        Returns:
            SimpleRAG or LongContextRAG: The appropriate RAG implementation.
        """
        return LongContextRAG(llm, args, path,**kargs)

class RAGManager:
    def __init__(self, llm: ByzerLLM, args: AutoCoderArgs, path: str):
        self.llm = llm
        self.args = args
        self.path = path
        self.rag = RAGFactory.get_rag(llm, args, path)

    def search(self, query: str) -> List[SourceCode]:
        """
        Perform a RAG search using the appropriate implementation.
        
        Args:
            query (str): The search query.
            
        Returns:
            List[SourceCode]: The search results.
        """
        return self.rag.search(query)

    def stream_chat_oai(
        self,
        conversations: List[Dict[str, Any]],
        model: Optional[str] = None,
        role_mapping: Optional[Dict[str, str]] = None,
        llm_config: Dict[str, Any] = {},
    ):
        """
        Perform a streaming chat using the appropriate RAG implementation.
        
        Args:
            conversations (List[Dict[str, Any]]): The conversation history.
            model (Optional[str]): The model to use for chat.
            role_mapping (Optional[Dict[str, str]]): Role mapping for the conversation.
            llm_config (Dict[str, Any]): Additional LLM configuration.
            
        Returns:
            Tuple containing the chat response generator and any additional context.
        """
        return self.rag.stream_chat_oai(conversations, model, role_mapping, llm_config)

    def build(self):
        """
        Build the RAG index using the appropriate implementation.
        """
        self.rag.build()



"""
Type definitions for Project Tracker module
"""

from pydantic import BaseModel
from typing import List, Dict, Any, Optional
from enum import Enum

class ExplorationMode(str, Enum):
    """Exploration modes for project tracking"""
    FULL_SCAN = "full_scan"  # Complete project scan
    AC_MODULES_ONLY = "ac_modules_only"  # Only scan AC modules
    TARGETED_SCAN = "targeted_scan"  # Target specific areas
    QUICK_OVERVIEW = "quick_overview"  # Quick project overview

class ModuleInfo(BaseModel):
    """Information about an AC module"""
    path: str
    name: str
    description: str
    dependencies: List[str] = []
    key_features: List[str] = []
    verification_commands: List[str] = []
    last_analyzed: Optional[str] = None

class ExplorationResult(BaseModel):
    """Result of project exploration"""
    success: bool
    mode: ExplorationMode
    modules_found: List[ModuleInfo]
    project_structure: Dict[str, Any]
    key_insights: List[str]
    recommendations: List[str]
    analysis_summary: str
    exploration_time: float
    error_message: Optional[str] = None

class ProjectTrackerRequest(BaseModel):
    """Request for project tracking operation"""
    mode: ExplorationMode = ExplorationMode.FULL_SCAN
    target_paths: Optional[List[str]] = None
    include_patterns: Optional[List[str]] = None
    exclude_patterns: Optional[List[str]] = None
    max_depth: Optional[int] = None
    custom_prompt: Optional[str] = None

class ProjectTrackerResponse(BaseModel):
    """Response from project tracking operation"""
    success: bool
    exploration_result: Optional[ExplorationResult] = None
    conversation_id: Optional[str] = None
    error_message: Optional[str] = None
    execution_time: float = 0.0 


"""
RuleFiles API 接口

提供所有公共 API 函数和便利函数。
"""

import os

# 导入第三方依赖
import byzerllm
from loguru import logger

from autocoder.common import AutoCoderArgs

# 导入核心组件
from .core.manager import AutocoderRulesManager
from .core.selector import RuleSelector
from .models.index import ConditionalRulesIndex
from .models.init_rule import InitRule

# 导入数据模型
from .models.rule_file import RuleFile
from .models.summary import AlwaysApplyRuleSummary

# 对外提供单例
_rules_manager = None


def get_rules(project_root: str | None = None) -> dict[str, str]:
    """获取所有规则文件内容，可指定项目根目录"""
    global _rules_manager
    if _rules_manager is None:
        _rules_manager = AutocoderRulesManager(project_root=project_root)
    return _rules_manager.get_rules()


def get_parsed_rules(project_root: str | None = None) -> list[RuleFile]:
    """获取所有解析后的规则文件，可指定项目根目录"""
    global _rules_manager
    if _rules_manager is None:
        _rules_manager = AutocoderRulesManager(project_root=project_root)
    return _rules_manager.get_parsed_rules()


def parse_rule_file(file_path: str, project_root: str | None = None) -> RuleFile:
    """解析指定的规则文件，可指定项目根目录"""
    global _rules_manager
    if _rules_manager is None:
        _rules_manager = AutocoderRulesManager(project_root=project_root)
    return _rules_manager.parse_rule_file(file_path)


def reset_rules_manager():
    """重置AutocoderRulesManager单例实例"""
    AutocoderRulesManager.reset_instance()
    global _rules_manager
    _rules_manager = None


def auto_select_rules(
    context: str,
    llm: byzerllm.ByzerLLM | None = None,
    args: AutoCoderArgs | None = None,
) -> dict[str, str]:
    """
    根据LLM的判断和规则元数据选择适用的规则。

    Args:
        context: 上下文信息
        llm: ByzerLLM 实例
        args: 可选的 AutoCoderArgs 参数

    Returns:
        Dict[str, str]: 选定规则的 {file_path: content} 字典
    """
    selector = RuleSelector(llm=llm, args=args)
    return selector.get_selected_rules_content(context=context)


def get_required_and_index_rules() -> dict[str, str]:
    """
    获取所有必须应用的规则文件(always_apply=True)和Index.md文件。

    Returns:
        Dict[str, str]: 包含必须应用的规则和Index.md文件的{file_path: content}字典。
    """
    # 获取所有解析后的规则文件
    parsed_rules = get_parsed_rules()
    result: dict[str, str] = {}
    logger.info(
        f"[RULES] 获取所有解析后的规则文件完成 | stage=load total_rules={len(parsed_rules)}"
    )

    for rule in parsed_rules:
        # 检查是否是always_apply=True的规则
        if rule.always_apply:
            result[rule.file_path] = rule.content
            logger.debug(
                f"[RULES] 添加必须应用的规则 | stage=load file_path={os.path.basename(rule.file_path)}"
            )

        # 检查是否是Index.md文件
        if os.path.basename(rule.file_path).lower() == "index.md":
            result[rule.file_path] = rule.content
            logger.debug(
                f"[RULES] 添加Index.md文件 | stage=load file_path={rule.file_path}"
            )

    logger.info(
        f"[RULES] 获取必须应用的规则和Index.md文件完成 | stage=load selected_count={len(result)}"
    )
    return result


def generate_always_apply_summary(
    llm: byzerllm.ByzerLLM | byzerllm.SimpleByzerLLM | None,
    args: AutoCoderArgs | None = None,
) -> AlwaysApplyRuleSummary | None:
    """
    便利函数：生成所有必须应用规则的合并摘要。

    Args:
        llm: ByzerLLM 实例
        args: 可选的 AutoCoderArgs 参数

    Returns:
        AlwaysApplyRuleSummary: 合并后的规则摘要，如果没有 LLM 则返回 None
    """
    selector = RuleSelector(llm=llm, args=args)
    return selector.generate_always_apply_summary()


def generate_conditional_rules_index(
    llm: byzerllm.ByzerLLM | byzerllm.SimpleByzerLLM | None,
    args: AutoCoderArgs | None = None,
) -> ConditionalRulesIndex | None:
    """
    便利函数：生成所有条件规则的索引目录。

    Args:
        llm: ByzerLLM 实例
        args: 可选的 AutoCoderArgs 参数

    Returns:
        ConditionalRulesIndex: 条件规则索引，如果没有 LLM 则返回 None
    """
    selector = RuleSelector(llm=llm, args=args)
    return selector.generate_conditional_rules_index()


def init_rule(
    llm: byzerllm.ByzerLLM | byzerllm.SimpleByzerLLM | None = None,
    args: AutoCoderArgs | None = None,
    project_root: str | None = None,
) -> InitRule | None:
    """
    使用 subagent 机制探索项目结构并生成初始化规则，自动保存到 .autocoderrules/init.md 文件。

    Args:
        llm: ByzerLLM 实例（可选，因为使用 subagent 机制）
        args: 可选的 AutoCoderArgs 参数
        project_root: 项目根目录，如果为 None 则使用当前目录

    Returns:
        InitRule: 生成的初始化规则对象，如果失败则返回 None
    """
    try:
        # 确定项目根目录
        if project_root is None:
            project_root = os.getcwd()

        logger.info(
            f"[RULES] 使用 subagent 机制开始生成初始化规则 | stage=init_rule project_root={project_root}"
        )

        # 使用 RuleSelector 生成初始化规则，subagent 机制会自动处理文件写入
        selector = RuleSelector(llm=llm, args=args)
        init_rule_result = selector.generate_init_rule(project_root=project_root)

        if init_rule_result is None:
            logger.error(
                "[RULES] 使用 subagent 机制生成初始化规则失败 | stage=init_rule"
            )
            return None

        logger.info(
            f"[RULES] 成功使用 subagent 机制生成初始化规则 | stage=init_rule file_path={init_rule_result.file_path} project_type={init_rule_result.project_type} technologies_count={len(init_rule_result.technologies)} commands_count={len(init_rule_result.commands)}"
        )
        logger.debug(
            f"[RULES] 初始化规则详情 | stage=init_rule technologies={', '.join(init_rule_result.technologies)} commands={', '.join(init_rule_result.commands)}"
        )

        return init_rule_result

    except Exception as e:
        logger.error(
            f"[RULES] 使用 subagent 机制生成初始化规则时出错 | stage=init_rule exc_type={type(e).__name__} error={e}",
            exc_info=True,
        )
        return None


def get_rules_for_conversation(project_root: str | None = None) -> str | None:
    """
    获取格式化的规则文本，用于添加到对话中。

    将必须规则（alwaysApply=true）的完整内容和条件规则（alwaysApply=false）的描述信息
    格式化为适合对话的文本格式。

    Args:
        project_root: 项目根目录，如果为 None 则使用当前目录

    Returns:
        str: 格式化的规则文本，如果没有规则则返回 None
    """
    try:
        # 获取所有解析后的规则
        all_rules = get_parsed_rules(project_root)

        if not all_rules:
            logger.info("[RULES] 未找到任何规则文件 | stage=load")
            return None

        # 分离必须规则和条件规则
        required_rules = [rule for rule in all_rules if rule.always_apply]
        conditional_rules = [rule for rule in all_rules if not rule.always_apply]

        # 构建消息内容
        message_parts = []

        # 添加必须规则的完整内容
        if required_rules:
            message_parts.append("## Required Rules (Always Applied)")
            message_parts.append("The following rules must always be followed:")
            message_parts.append("")

            for rule in required_rules:
                rule_name = os.path.basename(rule.file_path)
                message_parts.append(f"### {rule_name}")
                if rule.description:
                    message_parts.append(f"**Description**: {rule.description}")
                if rule.globs:
                    message_parts.append(
                        f"**Applicable Files**: {', '.join(rule.globs)}"
                    )
                message_parts.append("")
                message_parts.append(rule.content)
                message_parts.append("")

        # 添加条件规则的描述和路径
        if conditional_rules:
            message_parts.append("## Conditional Rules (Apply as Needed)")
            message_parts.append(
                "The following rules are available for specific contexts. Use the read_file tool to read their full content when needed:"
            )
            message_parts.append("")

            for rule in conditional_rules:
                rule_name = os.path.basename(rule.file_path)
                message_parts.append(f"### {rule_name}")
                message_parts.append(f"**Path**: `{rule.file_path}`")
                if rule.description:
                    message_parts.append(f"**Description**: {rule.description}")
                if rule.globs:
                    message_parts.append(
                        f"**Applicable Files**: {', '.join(rule.globs)}"
                    )
                message_parts.append("")

        # 如果没有任何规则，返回 None
        if not message_parts:
            return None

        combined_rules = "\n".join(message_parts)
        logger.info(
            f"[RULES] 成功格式化规则文本 | stage=load required_count={len(required_rules)} conditional_count={len(conditional_rules)}"
        )
        return combined_rules

    except Exception as e:
        logger.error(
            f"[RULES] 获取对话规则时出错 | stage=load exc_type={type(e).__name__} error={e}",
            exc_info=True,
        )
        return None



"""
Utility functions for handling async/sync conversions consistently.
"""

import asyncio
import concurrent.futures
from typing import TypeVar, Callable, Awaitable, Any
from functools import wraps

T = TypeVar('T')


def run_async_in_sync(async_func: Callable[..., Awaitable[T]], *args, **kwargs) -> T:
    """
    Run an async function in a synchronous context.
    
    This function handles both cases:
    1. When called from within an existing event loop (runs in a thread)
    2. When called from a synchronous context (creates new event loop)
    
    Args:
        async_func: The async function to run
        *args: Positional arguments for the function
        **kwargs: Keyword arguments for the function
        
    Returns:
        The result of the async function
    """
    def run_in_new_loop():
        """Run the async function in a new event loop."""
        loop = asyncio.new_event_loop()
        asyncio.set_event_loop(loop)
        try:
            result = loop.run_until_complete(async_func(*args, **kwargs))
            # Wait for all tasks to complete
            pending = asyncio.all_tasks(loop)
            if pending:
                loop.run_until_complete(asyncio.gather(*pending, return_exceptions=True))
            return result
        finally:
            # Cancel any remaining tasks
            pending = asyncio.all_tasks(loop)
            for task in pending:
                task.cancel()
            if pending:
                loop.run_until_complete(asyncio.gather(*pending, return_exceptions=True))
            loop.close()
    
    try:
        # Check if we're in an async context
        asyncio.get_running_loop()
        # We're in an async context, run in a thread
        with concurrent.futures.ThreadPoolExecutor() as executor:
            future = executor.submit(run_in_new_loop)
            return future.result()
    except RuntimeError:
        # No event loop running, we can run directly
        return run_in_new_loop()


def async_to_sync(async_func: Callable[..., Awaitable[T]]) -> Callable[..., T]:
    """
    Decorator to create a synchronous version of an async function.
    
    Usage:
        @async_to_sync
        async def my_async_func():
            await asyncio.sleep(1)
            return "done"
            
        # Can now be called synchronously
        result = my_async_func()
    """
    @wraps(async_func)
    def wrapper(*args, **kwargs):
        return run_async_in_sync(async_func, *args, **kwargs)
    return wrapper


class AsyncSyncMixin:
    """
    Mixin class that provides automatic sync versions of async methods.
    
    For any async method `foo`, this mixin automatically provides a `foo_sync` method.
    """
    
    def __getattr__(self, name: str) -> Any:
        if name.endswith('_sync'):
            async_name = name[:-5]  # Remove '_sync' suffix
            async_method = getattr(self, async_name, None)
            
            if async_method and asyncio.iscoroutinefunction(async_method):
                # Create a sync wrapper
                def sync_wrapper(*args, **kwargs):
                    return run_async_in_sync(async_method, *args, **kwargs)
                
                # Cache it for future calls
                setattr(self, name, sync_wrapper)
                return sync_wrapper
        
        raise AttributeError(f"'{self.__class__.__name__}' object has no attribute '{name}'") 


"""
Logging Plugin Example

记录工具调用日志的示例插件。
"""

import json
from typing import Optional, Dict, Any, TYPE_CHECKING
from autocoder.common.v2.agent.agentic_edit_types import BaseTool, ToolResult
from ..base_plugin import BaseToolCallPlugin
from ..plugin_interface import PluginPriority

if TYPE_CHECKING:
    from autocoder.common.v2.agent.agentic_edit import AgenticEdit


class LoggingPlugin(BaseToolCallPlugin):
    """
    日志记录插件
    
    记录所有工具调用的详细信息，包括参数、执行时间和结果。
    """
    
    def __init__(self, enabled: bool = True, detailed_logging: bool = False):
        """
        初始化日志插件
        
        Args:
            enabled: 插件是否启用
            detailed_logging: 是否记录详细日志（包括工具参数和结果内容）
        """
        super().__init__(enabled)
        self.detailed_logging = detailed_logging
        self.call_history = []
    
    @property
    def name(self) -> str:
        return "logging"
    
    @property
    def priority(self) -> PluginPriority:
        return PluginPriority.LOW  # 记录日志通常优先级较低
    
    @property 
    def description(self) -> str:
        return "Records detailed logs of all tool calls and executions"
    
    def before_tool_execution(
        self, 
        tool: BaseTool, 
        agent: Optional['AgenticEdit'],
        context: Optional[Dict[str, Any]] = None
    ) -> BaseTool:
        """记录工具执行前的信息"""
        if not self.enabled:
            return tool
        
        call_info = {
            "tool_name": type(tool).__name__,
            "timestamp": context.get("timestamp") if context else None,
            "call_id": context.get("call_id") if context else None,
            "phase": "before_execution"
        }
        
        if self.detailed_logging:
            call_info["tool_params"] = tool.model_dump() if hasattr(tool, 'model_dump') else str(tool)
        
        self.call_history.append(call_info)
        self.log_info(f"Starting execution of {type(tool).__name__}")
        
        if self.detailed_logging:
            self.log_info(f"Tool parameters: {json.dumps(call_info.get('tool_params', {}), indent=2)}")
        
        return tool
    
    def after_tool_execution(
        self, 
        tool: BaseTool, 
        tool_result: ToolResult, 
        agent: Optional['AgenticEdit'],
        context: Optional[Dict[str, Any]] = None
    ) -> ToolResult:
        """记录工具执行后的信息"""
        if not self.enabled:
            return tool_result
        
        call_info = {
            "tool_name": type(tool).__name__,
            "timestamp": context.get("timestamp") if context else None,
            "call_id": context.get("call_id") if context else None,
            "phase": "after_execution",
            "success": tool_result.success,
            "message": tool_result.message
        }
        
        if self.detailed_logging and tool_result.content:
            # 只记录前500个字符，避免日志过长
            content_str = str(tool_result.content)
            if len(content_str) > 500:
                content_str = content_str[:500] + "..."
            call_info["result_content"] = content_str
        
        self.call_history.append(call_info)
        
        status = "successfully" if tool_result.success else "with error"
        self.log_info(f"Completed execution of {type(tool).__name__} {status}")
        
        if not tool_result.success:
            self.log_warning(f"Tool execution failed: {tool_result.message}")
        
        if self.detailed_logging:
            self.log_info(f"Result: {json.dumps(call_info, indent=2, default=str)}")
        
        return tool_result
    
    def on_tool_error(
        self, 
        tool: BaseTool, 
        error: Exception, 
        agent: Optional['AgenticEdit'],
        context: Optional[Dict[str, Any]] = None
    ) -> Optional[ToolResult]:
        """记录工具执行错误"""
        if not self.enabled:
            return None
        
        call_info = {
            "tool_name": type(tool).__name__,
            "timestamp": context.get("timestamp") if context else None,
            "call_id": context.get("call_id") if context else None,
            "phase": "error",
            "error_type": type(error).__name__,
            "error_message": str(error)
        }
        
        self.call_history.append(call_info)
        self.log_error(f"Tool {type(tool).__name__} failed with {type(error).__name__}: {str(error)}")
        
        # 不提供恢复结果，只记录错误
        return None
    
    def get_call_history(self) -> list:
        """获取调用历史"""
        return self.call_history.copy()
    
    def clear_history(self) -> None:
        """清空调用历史"""
        self.call_history.clear()
        self.log_info("Call history cleared")
    
    def get_statistics(self) -> Dict[str, Any]:
        """获取统计信息"""
        total_calls = len([call for call in self.call_history if call["phase"] == "after_execution"])
        successful_calls = len([call for call in self.call_history 
                              if call["phase"] == "after_execution" and call.get("success", False)])
        error_calls = len([call for call in self.call_history if call["phase"] == "error"])
        
        return {
            "total_logged_calls": total_calls,
            "successful_calls": successful_calls,
            "error_calls": error_calls,
            "success_rate": successful_calls / total_calls if total_calls > 0 else 0.0,
            "detailed_logging": self.detailed_logging
        } 


"""
Restore manager for conversation data.

This module provides functionality to restore conversation data from backups,
including version management and data validation.
"""

import os
import json
import shutil
import threading
from datetime import datetime
from typing import Dict, List, Optional, Set, Tuple
from pathlib import Path

from ..exceptions import RestoreError, ConversationManagerError, BackupError
from ..config import ConversationManagerConfig
from .backup_manager import BackupManager, BackupMetadata


class RestoreManager:
    """
    Manages restore operations for conversation data.
    
    Provides functionality to restore data from full and incremental backups,
    with validation and version management.
    """
    
    def __init__(self, config: ConversationManagerConfig, backup_manager: BackupManager):
        """
        Initialize restore manager.
        
        Args:
            config: Configuration object containing restore settings
            backup_manager: BackupManager instance for accessing backup metadata
        """
        self.config = config
        self.backup_manager = backup_manager
        self.storage_path = Path(config.storage_path)
        self.backup_path = self.storage_path / "backups"
        self.temp_path = self.storage_path / "temp"
        
        # Ensure directories exist
        self.temp_path.mkdir(parents=True, exist_ok=True)
        
        # Thread lock for restore operations
        self._restore_lock = threading.Lock()
    
    def restore_conversation(
        self, 
        conversation_id: str, 
        backup_id: str,
        target_directory: Optional[str] = None
    ) -> bool:
        """
        Restore a specific conversation from backup.
        
        Args:
            conversation_id: ID of the conversation to restore
            backup_id: ID of the backup to restore from
            target_directory: Optional target directory for restoration
            
        Returns:
            True if restoration was successful
            
        Raises:
            RestoreError: If restoration fails
        """
        with self._restore_lock:
            try:
                # Get backup metadata
                metadata = self.backup_manager.get_backup_metadata(backup_id)
                if metadata is None:
                    raise RestoreError(f"Backup {backup_id} not found")
                
                # Verify backup integrity
                if not self.backup_manager.verify_backup(backup_id):
                    raise RestoreError(f"Backup {backup_id} integrity check failed")
                
                # Build complete backup chain for incremental backups
                backup_chain = self._build_backup_chain(backup_id)
                
                # Find conversation file in backup chain
                conversation_data = self._find_conversation_in_backups(
                    conversation_id, backup_chain
                )
                
                if conversation_data is None:
                    raise RestoreError(
                        f"Conversation {conversation_id} not found in backup {backup_id}"
                    )
                
                # Determine target directory
                if target_directory is None:
                    target_dir = self.storage_path / "conversations"
                else:
                    target_dir = Path(target_directory)
                
                target_dir.mkdir(parents=True, exist_ok=True)
                
                # Create backup of existing conversation if it exists
                existing_file = target_dir / f"{conversation_id}.json"
                if existing_file.exists():
                    backup_file = existing_file.with_suffix(
                        f".backup_{datetime.now().strftime('%Y%m%d_%H%M%S')}.json"
                    )
                    shutil.copy2(existing_file, backup_file)
                
                # Write restored conversation data
                with open(existing_file, 'w', encoding='utf-8') as f:
                    json.dump(conversation_data, f, indent=2, ensure_ascii=False)
                
                return True
                
            except Exception as e:
                raise RestoreError(f"Failed to restore conversation {conversation_id}: {str(e)}") from e
    
    def restore_full_backup(
        self, 
        backup_id: str,
        target_directory: Optional[str] = None,
        overwrite_existing: bool = False
    ) -> Dict[str, any]:
        """
        Restore all data from a full backup.
        
        Args:
            backup_id: ID of the backup to restore from
            target_directory: Optional target directory for restoration
            overwrite_existing: Whether to overwrite existing files
            
        Returns:
            Dictionary containing restore results
            
        Raises:
            RestoreError: If restoration fails
        """
        with self._restore_lock:
            try:
                # Get backup metadata
                metadata = self.backup_manager.get_backup_metadata(backup_id)
                if metadata is None:
                    raise RestoreError(f"Backup {backup_id} not found")
                
                if metadata.backup_type != "full":
                    raise RestoreError(f"Backup {backup_id} is not a full backup")
                
                # Verify backup integrity
                if not self.backup_manager.verify_backup(backup_id):
                    raise RestoreError(f"Backup {backup_id} integrity check failed")
                
                backup_dir = self.backup_path / backup_id
                
                # Determine target directory
                if target_directory is None:
                    target_dir = self.storage_path
                else:
                    target_dir = Path(target_directory)
                
                # Create timestamp for backup naming
                timestamp = datetime.now().strftime('%Y%m%d_%H%M%S')
                
                # Backup existing data if not overwriting
                existing_conversations_dir = target_dir / "conversations"
                existing_index_dir = target_dir / "index"
                
                backed_up_files = []
                if not overwrite_existing:
                    if existing_conversations_dir.exists():
                        backup_conversations_dir = target_dir / f"conversations.backup_{timestamp}"
                        shutil.copytree(existing_conversations_dir, backup_conversations_dir)
                        backed_up_files.append(str(backup_conversations_dir))
                    
                    if existing_index_dir.exists():
                        backup_index_dir = target_dir / f"index.backup_{timestamp}"
                        shutil.copytree(existing_index_dir, backup_index_dir)
                        backed_up_files.append(str(backup_index_dir))
                
                # Restore conversations
                restored_conversations = []
                conversations_backup_dir = backup_dir
                if conversations_backup_dir.exists():
                    target_conversations_dir = target_dir / "conversations"
                    target_conversations_dir.mkdir(parents=True, exist_ok=True)
                    
                    for conv_file in conversations_backup_dir.glob("*.json"):
                        target_file = target_conversations_dir / conv_file.name
                        shutil.copy2(conv_file, target_file)
                        restored_conversations.append(conv_file.stem)
                
                # Restore index files
                restored_indices = []
                index_backup_dir = backup_dir / "index"
                if index_backup_dir.exists():
                    target_index_dir = target_dir / "index"
                    target_index_dir.mkdir(parents=True, exist_ok=True)
                    
                    for index_file in index_backup_dir.glob("*.json"):
                        target_file = target_index_dir / index_file.name
                        shutil.copy2(index_file, target_file)
                        restored_indices.append(index_file.name)
                
                return {
                    "backup_id": backup_id,
                    "restore_timestamp": datetime.now().isoformat(),
                    "restored_conversations": restored_conversations,
                    "restored_indices": restored_indices,
                    "backed_up_files": backed_up_files,
                    "overwrite_existing": overwrite_existing
                }
                
            except Exception as e:
                raise RestoreError(f"Failed to restore full backup {backup_id}: {str(e)}") from e
    
    def restore_point_in_time(
        self, 
        target_timestamp: float,
        target_directory: Optional[str] = None
    ) -> Dict[str, any]:
        """
        Restore data to a specific point in time.
        
        Args:
            target_timestamp: Target timestamp to restore to
            target_directory: Optional target directory for restoration
            
        Returns:
            Dictionary containing restore results
            
        Raises:
            RestoreError: If restoration fails
        """
        with self._restore_lock:
            try:
                # Find the best backup for the target timestamp
                backup_id = self._find_best_backup_for_timestamp(target_timestamp)
                if backup_id is None:
                    raise RestoreError(f"No suitable backup found for timestamp {target_timestamp}")
                
                metadata = self.backup_manager.get_backup_metadata(backup_id)
                
                if metadata.backup_type == "full":
                    return self.restore_full_backup(
                        backup_id, 
                        target_directory=target_directory
                    )
                else:
                    # For incremental backups, we need to apply the backup chain
                    return self._restore_incremental_chain(
                        backup_id, 
                        target_directory=target_directory
                    )
                
            except Exception as e:
                raise RestoreError(f"Failed to restore to timestamp {target_timestamp}: {str(e)}") from e
    
    def list_restorable_conversations(self, backup_id: str) -> List[str]:
        """
        List conversations that can be restored from a backup.
        
        Args:
            backup_id: ID of the backup to check
            
        Returns:
            List of conversation IDs that can be restored
            
        Raises:
            RestoreError: If backup cannot be accessed
        """
        try:
            metadata = self.backup_manager.get_backup_metadata(backup_id)
            if metadata is None:
                raise RestoreError(f"Backup {backup_id} not found")
            
            if metadata.backup_type == "full":
                return metadata.conversation_ids or []
            else:
                # For incremental backups, get conversations from backup chain
                backup_chain = self._build_backup_chain(backup_id)
                all_conversations = set()
                
                for chain_backup_id in backup_chain:
                    chain_metadata = self.backup_manager.get_backup_metadata(chain_backup_id)
                    if chain_metadata and chain_metadata.conversation_ids:
                        all_conversations.update(chain_metadata.conversation_ids)
                
                return list(all_conversations)
                
        except Exception as e:
            raise RestoreError(f"Failed to list restorable conversations: {str(e)}") from e
    
    def validate_backup_chain(self, backup_id: str) -> Dict[str, any]:
        """
        Validate the integrity of a backup chain.
        
        Args:
            backup_id: ID of the backup to validate
            
        Returns:
            Dictionary containing validation results
            
        Raises:
            RestoreError: If validation fails
        """
        try:
            backup_chain = self._build_backup_chain(backup_id)
            validation_results = {
                "backup_id": backup_id,
                "chain_length": len(backup_chain),
                "chain_backups": [],
                "is_valid": True,
                "errors": []
            }
            
            for chain_backup_id in backup_chain:
                metadata = self.backup_manager.get_backup_metadata(chain_backup_id)
                if metadata is None:
                    validation_results["is_valid"] = False
                    validation_results["errors"].append(f"Backup {chain_backup_id} metadata not found")
                    continue
                
                # Verify backup integrity
                is_valid = self.backup_manager.verify_backup(chain_backup_id)
                
                backup_info = {
                    "backup_id": chain_backup_id,
                    "backup_type": metadata.backup_type,
                    "timestamp": metadata.timestamp,
                    "is_valid": is_valid
                }
                
                if not is_valid:
                    validation_results["is_valid"] = False
                    validation_results["errors"].append(f"Backup {chain_backup_id} integrity check failed")
                
                validation_results["chain_backups"].append(backup_info)
            
            return validation_results
            
        except Exception as e:
            raise RestoreError(f"Failed to validate backup chain: {str(e)}") from e
    
    def get_restore_preview(self, backup_id: str) -> Dict[str, any]:
        """
        Get a preview of what would be restored from a backup.
        
        Args:
            backup_id: ID of the backup to preview
            
        Returns:
            Dictionary containing restore preview information
            
        Raises:
            RestoreError: If preview generation fails
        """
        try:
            metadata = self.backup_manager.get_backup_metadata(backup_id)
            if metadata is None:
                raise RestoreError(f"Backup {backup_id} not found")
            
            backup_chain = self._build_backup_chain(backup_id)
            
            preview = {
                "backup_id": backup_id,
                "backup_type": metadata.backup_type,
                "backup_timestamp": metadata.timestamp,
                "backup_chain": backup_chain,
                "restorable_conversations": self.list_restorable_conversations(backup_id),
                "total_file_count": 0,
                "total_size_bytes": 0
            }
            
            # Calculate total size and file count for the chain
            for chain_backup_id in backup_chain:
                chain_metadata = self.backup_manager.get_backup_metadata(chain_backup_id)
                if chain_metadata:
                    preview["total_file_count"] += chain_metadata.file_count
                    preview["total_size_bytes"] += chain_metadata.total_size_bytes
            
            return preview
            
        except Exception as e:
            raise RestoreError(f"Failed to generate restore preview: {str(e)}") from e
    
    def _build_backup_chain(self, backup_id: str) -> List[str]:
        """
        Build the complete backup chain for a backup.
        
        Args:
            backup_id: ID of the backup to build chain for
            
        Returns:
            List of backup IDs in the chain, ordered from base to target
        """
        chain = []
        current_backup_id = backup_id
        
        while current_backup_id:
            metadata = self.backup_manager.get_backup_metadata(current_backup_id)
            if metadata is None:
                break
            
            chain.insert(0, current_backup_id)  # Insert at beginning
            
            if metadata.backup_type == "full":
                break  # Reached the base backup
            
            current_backup_id = metadata.base_backup_id
        
        return chain
    
    def _find_conversation_in_backups(
        self, 
        conversation_id: str, 
        backup_chain: List[str]
    ) -> Optional[Dict]:
        """
        Find conversation data in backup chain.
        
        Args:
            conversation_id: ID of the conversation to find
            backup_chain: List of backup IDs to search in
            
        Returns:
            Conversation data or None if not found
        """
        # Search from newest to oldest backup
        for backup_id in reversed(backup_chain):
            backup_dir = self.backup_path / backup_id
            conv_file = backup_dir / f"{conversation_id}.json"
            
            if conv_file.exists():
                try:
                    with open(conv_file, 'r', encoding='utf-8') as f:
                        return json.load(f)
                except Exception:
                    continue  # Try next backup in chain
        
        return None
    
    def _find_best_backup_for_timestamp(self, target_timestamp: float) -> Optional[str]:
        """
        Find the best backup for a target timestamp.
        
        Args:
            target_timestamp: Target timestamp
            
        Returns:
            Backup ID or None if no suitable backup found
        """
        backups = self.backup_manager.list_backups()
        
        # Find backups that are before or at the target timestamp
        suitable_backups = [
            b for b in backups
            if b.timestamp <= target_timestamp
        ]
        
        if not suitable_backups:
            return None
        
        # Return the most recent suitable backup
        best_backup = max(suitable_backups, key=lambda x: x.timestamp)
        return best_backup.backup_id
    
    def _restore_incremental_chain(
        self, 
        backup_id: str,
        target_directory: Optional[str] = None
    ) -> Dict[str, any]:
        """
        Restore from an incremental backup chain.
        
        Args:
            backup_id: ID of the incremental backup
            target_directory: Optional target directory
            
        Returns:
            Dictionary containing restore results
        """
        backup_chain = self._build_backup_chain(backup_id)
        
        if not backup_chain:
            raise RestoreError(f"Empty backup chain for {backup_id}")
        
        # Start with full backup restoration
        full_backup_id = backup_chain[0]
        results = self.restore_full_backup(
            full_backup_id,
            target_directory=target_directory,
            overwrite_existing=True
        )
        
        # Apply incremental backups in order
        for incremental_backup_id in backup_chain[1:]:
            incremental_results = self._apply_incremental_backup(
                incremental_backup_id,
                target_directory or str(self.storage_path)
            )
            
            # Merge results
            results["restored_conversations"].extend(
                incremental_results.get("restored_conversations", [])
            )
            results["restored_indices"].extend(
                incremental_results.get("restored_indices", [])
            )
        
        # Remove duplicates
        results["restored_conversations"] = list(set(results["restored_conversations"]))
        results["restored_indices"] = list(set(results["restored_indices"]))
        
        return results
    
    def _apply_incremental_backup(
        self, 
        backup_id: str,
        target_directory: str
    ) -> Dict[str, any]:
        """Apply an incremental backup to the target directory."""
        backup_dir = self.backup_path / backup_id
        target_dir = Path(target_directory)
        
        restored_conversations = []
        restored_indices = []
        
        # Apply conversation files
        for conv_file in backup_dir.glob("*.json"):
            target_file = target_dir / "conversations" / conv_file.name
            target_file.parent.mkdir(parents=True, exist_ok=True)
            shutil.copy2(conv_file, target_file)
            restored_conversations.append(conv_file.stem)
        
        # Apply index files
        index_backup_dir = backup_dir / "index"
        if index_backup_dir.exists():
            for index_file in index_backup_dir.glob("*.json"):
                target_file = target_dir / "index" / index_file.name
                target_file.parent.mkdir(parents=True, exist_ok=True)
                shutil.copy2(index_file, target_file)
                restored_indices.append(index_file.name)
        
        return {
            "restored_conversations": restored_conversations,
            "restored_indices": restored_indices
        } 


"""
InitRule 数据模型

用于表示项目初始化规则的生成结果模型。
"""

from pydantic import BaseModel, Field


class InitRule(BaseModel):
    """项目初始化规则"""

    content: str = Field(description="生成的初始化规则内容")
    project_type: str = Field(description="识别的项目类型")
    commands: list[str] = Field(default_factory=list, description="检测到的命令列表")
    technologies: list[str] = Field(default_factory=list, description="检测到的技术栈")
    file_path: str = Field(description="保存的文件路径")
